/*
 * Version: $RCSfile: VolaSmsPlus.cs,v $ - $Revision: 1.3 $ - $Date: 2009/02/17 09:32:30 $
 *   
 *  This file is part of VolaSmsPlus.
 *
 *   VolaSmsPlus is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published by
 *   the Free Software Foundation, either version 3 of the License.
 * 
 *   VolaSmsPlus is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with VolaSmsPlus.  If not, see <http://www.gnu.org/licenses/>.
**/


using System;
using System.Text;
using System.Web;
using System.Reflection;
using System.Text.RegularExpressions;
using System.Collections;

namespace Vola.VolaSmsPlus
{
    /// <summary>
    /// 
    /// </summary>
    public class VolaSmsPlus
    {
        private const int MAX_DIM_ARRAY = 1000;
        private const int MAX_DIM_ARRAY_SENDER = 30;

        private string _UserName;
        private string _Password;
        private bool _Notify;
        private bool _LongMessage;
        private bool _EncryptSSL;
        private bool _AutomaticProxy;
        private string _ProxyHost;
        private string _ProxyPort;
        private string _ProxyUsername;
        private string _ProxyPassword;
        private string _VapCode;

        private ArrayList _MessageListToSend;
        private ArrayList _MessageListToNotify;
        private ArrayList _ListSender;

        private Boolean _TestMode;
        private string _DefaultCountry;
       
        /// <summary>
        /// Costruttore
        /// </summary>
        public VolaSmsPlus()
        {
            _TestMode = false;
            _MessageListToSend = new ArrayList(MAX_DIM_ARRAY);
            _MessageListToNotify = new ArrayList(MAX_DIM_ARRAY);
            _ListSender = new ArrayList(MAX_DIM_ARRAY_SENDER);
            _DefaultCountry = "+39";
        }

        #region propriet per la gestione di VolaSmsPlus
        /// <summary>
        /// Username dell'account Vola
        /// </summary>
        public string UserName
        {
            get
            {
                return _UserName;
            }
            set
            {                
                _UserName = value;
            }
        }
        /// <summary>
        /// Password dell'account Vola
        /// </summary>
        public string Password
        {
            get
            {
                return _Password;
            }
            set
            {
               
                _Password = value;             
            }
        }

        /// <summary>
        /// Abilita/Disabilita il servizio di notifica SMS per il prossimo invio
        /// </summary>
        public bool Notify
        {
            get
            {
                return _Notify;
            }
            set
            {
                _Notify = value;
            }
        }     
        /// <summary>
        /// Abilita/Disabilita il servizio per poter inviare messaggi lunghi
        /// </summary>
        public bool LongMessage
        {
            get
            {
                return _LongMessage;
            }
            set
            {
                _LongMessage = value;
            }
        }
        /// <summary>
        /// Abilita/Disabilita utilizzo di un server proxy per la connessione al gateway Vola
        /// </summary>
        public bool AutomaticProxy
        {
            get
            {
                return _AutomaticProxy;
            }
            set
            {
                _AutomaticProxy = value;
            }
        }
        /// <summary>
        /// Abilita/Disabilita la connessione sicura al gateway Vola
        /// </summary>
        public bool EncryptSSL
        {
            get
            {
                return _EncryptSSL;
            }
            set
            {
                _EncryptSSL = value;
            }
        }
        /// <summary>
        /// Nome o Indirizzo dell'host per la connessione internet
        /// </summary>
        public string ProxyHost
        {
            get
            {
                return _ProxyHost;
            }
            set
            {
                _ProxyHost = value;
            }
        }
        /// <summary>
        /// Porta dell'host per la connessione internet
        /// </summary>
        public string ProxyPort
        {
            get
            {
                return _ProxyPort;
            }
            set
            {
                _ProxyPort = value;
            }
        }
        /// <summary>
        /// Eventuale Username dell'host per la connessione internet
        /// </summary>
        public string ProxyUsername
        {
            get
            {
                return _ProxyUsername;
            }
            set
            {
                _ProxyUsername = value;
            }
        }
        /// <summary>
        /// Eventuale Password dell'host per la connessione internet
        /// </summary>
        public string ProxyPassword
        {
            get
            {
                return _ProxyPassword;
            }
            set
            {
                _ProxyPassword = value;
            }
        }
        /// <summary>
        /// Abilita/Disabilita la funzionalit test per invio SMS
        /// </summary>        
        public Boolean TestMode
        {
            get
            {
                return _TestMode;
            }
            set
            {
                _TestMode = value;
            }
        }
        /// <summary>
        /// Questo parametro deve essere utilizzato per chi ha aderito al programma VAP (Vola Affiliate Program).
        /// Deve essere valorizzato con il codice alfanumerico (VAP CODE) assegnato nel momento della registrazione.
        /// Per maggiori informazioni inviare una mail a supporto@vola.it
        /// </summary>
        public string VapCode{
            get
            {
                return _VapCode;
            }
            set
            {
                if (value.Length >10 || value.Contains(" "))
                    throw new VolaException("Il codice VAPCODE  errato.",VolaError.VOLASMSPLUS_ERROR_VAPCODE);                 
                _VapCode = value.Trim();
            }
        }
        /// <summary>
        /// Prefisso internazionale di default
        /// </summary>
        public string DefaultCountry
        {
            get
            {
                return _DefaultCountry;
            }
            set
            {
                _DefaultCountry = value;
            }
        }
        #endregion


        #region Metodi per la gestione del profilo Vola
        /// <summary>
        /// Comando Vola restituisce le informazioni sull'account Vola.
        /// </summary>
        /// <returns>Ritorna oggetto VolaProfiloUtente inizializzato con le informazioni sull'account Vola</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>             
        public VolaProfiloUtente GetProfile()
        {
            string Risposta = "";

            try
            {
                                
                //Make the url request
                string urlPath = "CMD=18&UID=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) +
                                 "&PWD=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) +
                                 "&SERIAL=" + Vola.VolaSmsPlus.VolaConstant.SerialCode +
                                 "&VCODE=" + _VapCode;

                //Invia richiesta               
                Risposta = VolaUtility.InvioDatiServer(urlPath,_EncryptSSL,_AutomaticProxy,_ProxyHost,_ProxyPort,_ProxyUsername,_ProxyPassword);
                
                //Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta);

                //Parse Profile
                string[] ProfiloSplit = Risposta.Split(VolaConstant.SeparatorProfilo1);
                VolaProfiloUtente Profilo = new VolaProfiloUtente();

                //Riempo la struttura dati
                Profilo.profilo = ProfiloSplit[0];
                Profilo.CreditoSms = ProfiloSplit[1];
                Profilo.CreditoMms = ProfiloSplit[2];
                Profilo.LicenzaVolaOffice = ProfiloSplit[3];
                Profilo.MittentePredefinito = ProfiloSplit[4];
                if (ProfiloSplit[5] == "S")
                    Profilo.ServizioNotificheUtente = true;
                else
                    Profilo.ServizioNotificheUtente = false;
                if (ProfiloSplit[6] == "S")
                    Profilo.ServizioNotificheServer = true;
                else
                    Profilo.ServizioNotificheServer = false;
                if (ProfiloSplit[7] == "S")
                    Profilo.ServizioEstero = true;
                else
                    Profilo.ServizioEstero = false;
               
                if (ProfiloSplit[8].Length > 0 && ProfiloSplit[8].ToUpper() != "null".ToUpper())
                    Profilo.ChiaviRicezione = ProfiloSplit[8].Split(VolaConstant.SeparatorProfilo2);
                else
                    Profilo.ChiaviRicezione = new string[0];
                Profilo.MittentiVola = ProfiloSplit[9];
                Profilo.MittentiDisabilitati = ProfiloSplit[10];
                Profilo.DataScadenzaCredito = ProfiloSplit[11];
                Profilo.ListaMittenti = ProfiloSplit[12];
                return Profilo;                                
            }
            catch (VolaException ex)
            {
                throw new VolaException(ex.Messaggio, ex.Codice);
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore grave durante la connessione al servizio SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);
            }
        }

        /// <summary>
        /// Comando Vola restituisce il numero credito residuo di VolaSMS. Il credito e' espresso in numero di messaggi.
        /// </summary>
        /// <returns>Ritorna un valore stringa con il credito residuo dell'account Vola</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>      
        public string GetCreditSMS()
        {
            string Risposta = "";

            try
            {
                                
                //Make the url request
                string urlPath = "CMD=1&UID=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) +
                                 "&PWD=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) +
                                 "&SERIAL=" + Vola.VolaSmsPlus.VolaConstant.SerialCode;
                if (_VapCode.Length > 0)
                    urlPath = urlPath + "&VCODE=" + _VapCode;

                //Invia richiesta               
                Risposta = VolaUtility.InvioDatiServer(urlPath,_EncryptSSL,_AutomaticProxy,_ProxyHost,_ProxyPort,_ProxyUsername,_ProxyPassword);
                
                //Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta);

                return Risposta;
            }
            catch (VolaException ex)
            {
                throw new VolaException(ex.Messaggio, ex.Codice);
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore grave durante la connessione al servizio SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);
                
            }
        }
        #endregion

        #region Metodi e propriet per InvioSMS
       
        /// <summary>
        /// Lista dei messaggi da inviare o inviati al gateway Vola
        /// </summary>
        public ArrayList MessageListToSend
        {
            get
            {
                return _MessageListToSend;
            }
            set
            {
                _MessageListToSend = value;
            }
        }

        /// <summary>
        /// Inserisce un nuovo messaggio nella lista dei messaggi da inviare al gateway Vola
        /// </summary>
        /// <param name="Gsm">Destinatario del messaggio (Numero Gsm)</param>
        /// <param name="Testo">Testo del messaggio e NON pu contenere i caratteti 'tab' e 'cr_lf'.
        /// Se il testo del messaggio supera i 160 caratteri il gateway di VolaSMS effettua uno split del testo
        /// in modo che ad ogni destinatario vengano inviati N messaggi ciascuno lungo al massimo 160 caratteri.
        /// Il numero massimo di caratteri che pu contenere il campo messaggio e' 1440 caratteri.</param>
        /// <param name="Mittente">Mittente del gruppo di invio. E' il mittente con cui verra' inviato l'sms al destinatario o ai destinatari del messaggio.
        /// Il campo avr effetto solo se l'account utilizzata supporta l'invio di messaggi con campo mittente modificabile. 
        /// In quest'ultimo caso il gateway di VolaSMS utilizzer il valore di default vola.it o SMS, nel caso in cui il profilo di listino impostato lo supporti.
        /// Il campo pu contenere al max 11 caratteri alfanumerici, oppure 14 caratteri numerici.
        /// Nel caso in cui il campo non sia correttamente valorizzato il gateway restituir un errore.</param>
        /// <returns>Il messaggio inserito nella lista dei messaggi da inviare al gateway Vola</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>            
        public VolaMessaggio AddMessageToSend(string Gsm, string Testo, string Mittente)
        {
            return AddMessageToSend(Gsm, Testo, Mittente, DateTime.MinValue);
        }

        /// <summary>
        /// Inserisce un nuovo messaggio nella lista dei messaggi da inviare al gateway Vola
        /// </summary>
        /// <param name="Gsm">Destinatario del messaggio (Numero Gsm)</param>
        /// <param name="Testo">Testo del messaggio e NON pu contenere i caratteti 'tab' e 'cr_lf'.
        /// Se il testo del messaggio supera i 160 caratteri il gateway di VolaSMS effettua uno split del testo
        /// in modo che ad ogni destinatario vengano inviati N messaggi ciascuno lungo al massimo 160 caratteri.
        /// Il numero massimo di caratteri che pu contenere il campo messaggio e' 1440 caratteri.</param>
        /// <param name="Mittente">Mittente del gruppo di invio. E' il mittente con cui verra' inviato l'sms al destinatario o ai destinatari del messaggio.
        /// Il campo avr effetto solo se l'account utilizzata supporta l'invio di messaggi con campo mittente modificabile. 
        /// In quest'ultimo caso il gateway di VolaSMS utilizzer il valore di default vola.it o SMS, nel caso in cui il profilo di listino impostato lo supporti.
        /// Il campo pu contenere al max 11 caratteri alfanumerici, oppure 14 caratteri numerici.
        /// Nel caso in cui il campo non sia correttamente valorizzato il gateway restituir un errore.</param>
        /// <param name="DataOra">Data e ora in cui il messaggio deve essere inviato</param>
        /// <returns>Il messaggio inserito nella lista dei messaggi da inviare al gateway Vola</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>                   
        public VolaMessaggio AddMessageToSend(string Gsm, string Testo, string Mittente, DateTime DataOra)
        {
            string[] listaGsm = new string[1];
            listaGsm[0] = Gsm;
            return AddMessageToSend(listaGsm, Testo, Mittente, DataOra);

        }

        /// <summary>
        /// Inserisce un nuovo messaggio nella lista dei messaggi da inviare al gateway Vola
        /// </summary>
        /// <param name="Gsm">Lista dei destinatari del messaggio (Numero Gsm)</param>
        /// <param name="Testo">Testo del messaggio e NON pu contenere i caratteti 'tab' e 'cr_lf'.
        /// Se il testo del messaggio supera i 160 caratteri il gateway di VolaSMS effettua uno split del testo
        /// in modo che ad ogni destinatario vengano inviati N messaggi ciascuno lungo al massimo 160 caratteri.
        /// Il numero massimo di caratteri che pu contenere il campo messaggio e' 1440 caratteri.</param>
        /// <param name="Mittente">Mittente del gruppo di invio. E' il mittente con cui verra' inviato l'sms al destinatario o ai destinatari del messaggio.
        /// Il campo avr effetto solo se l'account utilizzata supporta l'invio di messaggi con campo mittente modificabile. 
        /// In quest'ultimo caso il gateway di VolaSMS utilizzer il valore di default vola.it o SMS, nel caso in cui il profilo di listino impostato lo supporti.
        /// Il campo pu contenere al max 11 caratteri alfanumerici, oppure 14 caratteri numerici.
        /// Nel caso in cui il campo non sia correttamente valorizzato il gateway restituir un errore.</param>
        /// <returns>Il messaggio inserito nella lista dei messaggi da inviare al gateway Vola</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>            
        public VolaMessaggio AddMessageToSend(string[] Gsm, string Testo, string Mittente)
        {
            return AddMessageToSend(Gsm, Testo, Mittente, DateTime.MinValue);
        }

        /// <summary>
        /// Inserisce un nuovo messaggio nella lista dei messaggi da inviare al gateway Vola
        /// </summary>
        /// <param name="Gsm">Lista dei destinatari del messaggio (Numero Gsm)</param>
        /// <param name="Testo">Testo del messaggio e NON pu contenere i caratteti 'tab' e 'cr_lf'.
        /// Se il testo del messaggio supera i 160 caratteri il gateway di VolaSMS effettua uno split del testo
        /// in modo che ad ogni destinatario vengano inviati N messaggi ciascuno lungo al massimo 160 caratteri.
        /// Il numero massimo di caratteri che pu contenere il campo messaggio e' 1440 caratteri.</param>
        /// <param name="Mittente">Mittente del gruppo di invio. E' il mittente con cui verra' inviato l'sms al destinatario o ai destinatari del messaggio.
        /// Il campo avr effetto solo se l'account utilizzata supporta l'invio di messaggi con campo mittente modificabile. 
        /// In quest'ultimo caso il gateway di VolaSMS utilizzer il valore di default vola.it o SMS, nel caso in cui il profilo di listino impostato lo supporti.
        /// Il campo pu contenere al max 11 caratteri alfanumerici, oppure 14 caratteri numerici.
        /// Nel caso in cui il campo non sia correttamente valorizzato il gateway restituir un errore.</param>
        /// <param name="DataOra">Data e ora in cui il messaggio deve essere inviato</param>
        /// <returns>Il messaggio inserito nella lista dei messaggi da inviare al gateway Vola</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>                  
        public VolaMessaggio AddMessageToSend(string[] Gsm, string Testo, string Mittente, DateTime DataOra)
        {
            try
            {
                VolaMsdn GsmOK;
                string TestoOK;
                string MittenteOK;
                string DataOraOK;

                VolaMsdnStatus stato = VolaMsdnStatus.MSDN_INLIST;

                //Trasformo il carattere ritorno a capo
                TestoOK = Testo.Replace("\r\n", "\n");
                //Verifico caratteri non permessi           
                string caract = VolaConstant.CaractTextOK;
                Regex emailregex = new Regex(caract);
                Match m = emailregex.Match(Testo);
                if (m.Success == false)
                    stato = VolaMsdnStatus.MSDN_ERRTEXT;

                //Verifico se il testo  troppo lungo considerando i caratteri che valgono doppio
                string buffer = TestoOK;
                string[] CaratDouble = VolaConstant.CaractDouble.Split(',');
                for (int i = 0; i < CaratDouble.Length; i++)
                    buffer = buffer.Replace(CaratDouble[i], "xx");

                if (LongMessage)
                {
                    if (buffer.Length > 1440)
                        stato = VolaMsdnStatus.MSDN_ERRTEXT;
                }else{
                    if (buffer.Length > 160)
                        stato = VolaMsdnStatus.MSDN_ERRTEXT;
                }
                              
                //Verifico il mittente
                MittenteOK = Mittente;
                if (MittenteOK.Length < 1 || MittenteOK.Length > 15)
                    stato = VolaMsdnStatus.MSDN_ERRSENDER;

                //Verifico la data
                if (DataOra == DateTime.MinValue || DataOra == DateTime.MaxValue)
                    DataOraOK = "0000-00-00" + VolaConstant.SeparatorBlock + "00:00";
                else                    
                    DataOraOK = DataOra.ToString("yyyy-MM-dd") + VolaConstant.SeparatorBlock + DataOra.ToString("HH:mm");
                DataOraOK = DataOraOK.Replace(".", ":");

                //Verifico la capacit della lista dei messaggi
                if (_MessageListToSend.Count >= 1000)
                    stato = VolaMsdnStatus.MSDN_ERRDATETIME;

                //Creo un nuovo messaggio da inviare
                VolaMessaggio NuovoMessaggio = new VolaMessaggio();

                //Verifico i destinari                
                for (int i = 0; i < Gsm.Length; i++)
                {
                    GsmOK = new VolaMsdn();
                    GsmOK.gsm = Gsm[i];
                    GsmOK.gsm = VolaCheckGsm.CheckGSMNumber(GsmOK.gsm,VolaCheckGsm.FormatGsm.PLUS_COUNTRY_MSDN);
                    if (GsmOK.gsm.Length == 0)
                        GsmOK.status = VolaMsdnStatus.MSDN_ERRPHONEGSM;
                    else
                        GsmOK.status = stato;
                    NuovoMessaggio.listaDestinatari.Add(GsmOK);
                }

                //Inizializzo il messaggio
                NuovoMessaggio.testo = TestoOK;
                NuovoMessaggio.mittente = MittenteOK;
                NuovoMessaggio.dataora = DataOraOK;
                NuovoMessaggio.cid = _MessageListToSend.Count;

                //Aggiungo il nuovo messaggio alla lista
                _MessageListToSend.Add(NuovoMessaggio);
                return NuovoMessaggio;
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }


        }

        /// <summary>
        /// Rimuove un messaggio dalla lista dei messaggi da inviare al gateway Vola
        /// </summary>
        /// <param name="index">Indice del messaggio da rimuovere</param>
        /// <returns>True in caso positivo, False altrimenti</returns>
        public bool RemoveMessageToSend(int index)
        {        
            try{
                if (index >= _MessageListToSend.Count)
                    return false;

                //Rimuove il messaggio dalla lista
                _MessageListToSend.RemoveAt(index);
                return true;
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }

        }

        /// <summary>
        /// Svuota la lista dei messaggi da inviare al gateway Vola
        /// </summary>
        /// <returns>True in caso positivo, False altrimenti</returns>
        public bool ClearMessageToSend()
        {
            try{
                //Cancella tutta la lista
                _MessageListToSend.Clear();
                return true;
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }

        /// <summary>
        /// Restituisce il numero di messaggi contenuti nella lista dei messaggi da inviare al gateway Vola
        /// </summary>
        /// <returns>Il numero di messaggi contenuti nella lista dei messaggi da inviare al gateway Vola</returns>
        public int CountMessageToSend()
        {
            try{
                //Numero messaggi nella lista
                return _MessageListToSend.Count;
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }

        /// <summary>
        /// Restituisce il messaggio inserito nella lista dei messaggi da inviare al gateway Vola alla posizione indicata
        /// </summary>
        /// <param name="index">Indice del messaggio nella lista</param>
        /// <returns>Il messaggio inserito nella lista dei messaggi da inviare al gateway Vola alla posizione indicata</returns>
        public VolaMessaggio GetMessageToSend(int index)
        {
            try{
                IEnumerator Lista = _MessageListToSend.GetEnumerator(index, 1);
                Lista.MoveNext();
                return (VolaMessaggio) Lista.Current;           
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }

        /// <summary>
        /// Comando Vola consente l'invio di messaggi multipli in una unica richiesta, diminuendo cosi' il numero di connessioni tra client e server ed aumentando la performance di comunicazione.
        /// </summary>
        /// <returns>True in caso positivo, False altrimenti</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>                         
        public bool SendMessageToSend()
        {
            try{
                 
                VolaMessaggio NuovoMessaggio;
                VolaMsdn NuovoMsdn;
                string Risposta;

                //Controllo se la lista  vuota
                if (_MessageListToSend.Count ==0)
                    return false;

                //Make the url request
                string urlPath = "CMD=44&UID=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) +
                                 "&PWD=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) +
                                 "&SERIAL=" + Vola.VolaSmsPlus.VolaConstant.SerialCode;
                if (_VapCode.Length > 0)
                    urlPath = urlPath + "&VCODE=" + _VapCode;
                if (_Notify)
                    urlPath = urlPath + "&NOTIFY=1";
                else
                    urlPath = urlPath +"&NOTIFY=0";
                if (_TestMode)
                    urlPath = urlPath + "&TEST=1";
                else
                    urlPath = urlPath + "&TEST=0";
                urlPath = urlPath + "&SENDDATA=";

                //Ciclo sui messaggi nella lista
                IEnumerator Lista = _MessageListToSend.GetEnumerator();
                while (Lista.MoveNext())
                {
                    
                    NuovoMessaggio = (VolaMessaggio) Lista.Current;
                    urlPath = urlPath + NuovoMessaggio.cid.ToString().Trim() + VolaConstant.SeparatorBlock;
                    urlPath = urlPath + VolaUtility.UrlEncode(NuovoMessaggio.mittente) + VolaConstant.SeparatorBlock;

                    //Ciclo sui destinatari del messaggio
                    IEnumerator ListaMsdn = NuovoMessaggio.listaDestinatari.GetEnumerator();
                    while (ListaMsdn.MoveNext())
                    {
                        NuovoMsdn = (VolaMsdn)ListaMsdn.Current;
                        if (NuovoMsdn.status == VolaMsdnStatus.MSDN_INLIST)
                            urlPath = urlPath + VolaUtility.UrlEncode(NuovoMsdn.gsm) + VolaConstant.SeparatorPhone;
                    }
                    //Elimino l'ultmi due caratteri
                    if (urlPath.Substring(urlPath.Length - 1, 1) == VolaConstant.SeparatorPhone)
                        urlPath = urlPath.Substring(0, urlPath.Length - 1);
                    
                    //Aggiunfo gli altri parametri del messaggio
                    urlPath = urlPath + VolaConstant.SeparatorBlock;
                    UTF8Encoding e = new UTF8Encoding();
                    urlPath = urlPath + VolaUtility.UrlEncode(NuovoMessaggio.testo) + VolaConstant.SeparatorBlock;
                    urlPath = urlPath + VolaUtility.UrlEncode(NuovoMessaggio.dataora);
                    urlPath = urlPath + VolaConstant.SeparatorGroup;
                }

                //Elimino l'ultmi due caratteri
                urlPath.Substring(0, urlPath.Length - 2);

                //Invia richiesta               
                Risposta = VolaUtility.InvioDatiServer(urlPath,_EncryptSSL,_AutomaticProxy,_ProxyHost,_ProxyPort,_ProxyUsername,_ProxyPassword);
                
                //Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta);
          
                string sListaOrderdID = "";
                string[] messaggio;
                string[] messaggi = Risposta.Split(VolaConstant.SeparatorRespost1);
                //Aggiorno lo stato dei messaggi con la risposta del gateway
                for (int i = 0; i < messaggi.Length; i++)
                {
                    messaggio = messaggi[i].Split(VolaConstant.SeparatorRespost2);
                    if (messaggio[1].Length > 0 && messaggio[1] != "0")
                        sListaOrderdID = sListaOrderdID + messaggio[1].Replace(",", ";") + ";";

                    SetStatusMessage(messaggio[0], messaggio[2], messaggio[1]);
                }
                
                 //Invio la conferma al gateway sull'invio avvenuto in modo corretto
                if (sListaOrderdID.Length > 0)
                    return ConfirmSendMessage(sListaOrderdID);
                else
                    return true;

            }
            catch (VolaException ex)
            {
                throw new VolaException(ex.Messaggio, ex.Codice);
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }

        }

        /// <summary>
        /// Aggiorno lo stato dei messaggi con la risposta del gateway
        /// </summary>
        /// <param name="sCID">ID identificativo</param>
        /// <param name="PhoneGsmKO">Numero GSM del destinatario</param>
        /// <param name="sOrderId">ID identificativo</param>
        private void SetStatusMessage(string sCID,string PhoneGsmKO,string sOrderId)
        {
            VolaMsdn NuovoMsdn;
            int CID = Int32.Parse(sCID);     
       
            VolaMessaggio Messaggio = (VolaMessaggio) _MessageListToSend[CID];
            Messaggio.orderid = sOrderId.Split(VolaConstant.SeparatorRespost3);

            IEnumerator ListaMsdn = Messaggio.listaDestinatari.GetEnumerator();            
            while (ListaMsdn.MoveNext())
            {
                NuovoMsdn = (VolaMsdn)ListaMsdn.Current;

                if (PhoneGsmKO.IndexOf(NuovoMsdn.gsm) == -1)
                    NuovoMsdn.status = VolaMsdnStatus.MSDN_SENT;               
                else
                    NuovoMsdn.status = VolaMsdnStatus.MSDN_ERRGATEWAY;                
            }
        }

        /// <summary>
        /// Comando Vola per la conferma dell'invio dei messaggi contenuti nalla lista dei messaggi da inviare al gateway Vola 
        /// </summary>
        /// <param name="sListaOrderID">Lista ID identificati dei messaggi inviati correttamente</param>
        /// <returns>True in caso positivo, False altrimenti</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>                                 
        private bool ConfirmSendMessage(string sListaOrderID)
        {
            string Risposta = "";

            try
            {
               //Make the url request
                string urlPath = "CMD=45&UID=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) +
                                 "&PWD=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) +
                                 "&SERIAL=" + Vola.VolaSmsPlus.VolaConstant.SerialCode +
                                 "&ORDERID=" + sListaOrderID;

                if (_VapCode.Length > 0)
                    urlPath = urlPath + "&VCODE=" + _VapCode;

                //Invia richiesta               
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _ProxyPassword);

                //Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta);

                return true;
            }
            catch (VolaException ex)
            {
                throw new VolaException(ex.Messaggio, ex.Codice);
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore grave durante la connessione al servizio SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }
        #endregion

        #region Metodi per la Gestione delle notifiche
        /// <summary>
        /// Lista dei messaggi per ricevere lo stato di notifica
        /// </summary>
        public ArrayList MessageListToNotify
        {
            get
            {
                return _MessageListToNotify;
            }
            set
            {
                _MessageListToNotify = value;
            }
        }

        /// <summary>
        /// Inserisce un nuovo messaggio nella lista dei messaggi per ricevere lo stato di notifica
        /// </summary>
        /// <param name="OrderId">Numero identificativo sms</param>
        /// <returns>Il messaggio inserito nella lista</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>            
        public VolaMessaggio AddMessageToNotify(string OrderId)
        {            
            return AddMessageToNotify(OrderId, "");
        }

        /// <summary>
        /// Inserisce un nuovo messaggio nella lista dei messaggi per ricevere lo stato di notifica
        /// </summary>
        /// <param name="OrderId">Numero identificativo sms</param>
        /// <param name="Gsm">Destinatario del messaggio (Numero Gsm)</param>
        /// <returns>Il messaggio inserito nella lista</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>    
        public VolaMessaggio AddMessageToNotify(string OrderId, string Gsm)
        {
            try
            {
                VolaMsdn GsmOK = new VolaMsdn();
                string OrderIdOK;

                if (OrderId.Length == 0 && OrderId.Length<10)
                    return null;
                OrderIdOK = OrderId;

                //Verifico il destinario
                GsmOK.gsm = Gsm;                    
                GsmOK.gsm = VolaCheckGsm.CheckGSMNumber(GsmOK.gsm, VolaCheckGsm.FormatGsm.PLUS_COUNTRY_MSDN);              
                GsmOK.status = VolaMsdnStatus.MSDN_INLIST;
                                    
                //Verifico la capacit della lista dei messaggi
                if (_MessageListToNotify.Count >= 1000)
                    return null;

                //Creo un nuovo messaggio da notificare
                VolaMessaggio NuovoMessaggio = new VolaMessaggio();
                NuovoMessaggio.listaDestinatari.Add(GsmOK);
                NuovoMessaggio.orderid = new string[1];
                NuovoMessaggio.orderid[0] = OrderIdOK;
                
                //Aggiungo il nuovo messaggio alla lista
                _MessageListToNotify.Add(NuovoMessaggio);
                return NuovoMessaggio;
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }

        }

        /// <summary>
        /// Rimuove un messaggio dalla lista dei messaggi per ricevere lo stato di notifica
        /// </summary>
        /// <param name="index">Indice del messaggio da rimuovere</param>
        /// <returns>True in caso positivo, False altrimenti</returns>
        public bool RemoveMessageToNotify(int index)
        {
            try
            {
                if (index >= _MessageListToNotify.Count)
                    return false;

                //Rimuove il messaggio dalla lista
                _MessageListToNotify.RemoveAt(index);
                return true;
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }

        }

        /// <summary>
        /// Svuota la lista dei messaggi per ricevere lo stato di notifica
        /// </summary>
        /// <returns>True in caso positivo, False altrimenti</returns>
        public bool ClearMessageToNotify()
        {
            try
            {
                //Cancella tutta la lista
                _MessageListToNotify.Clear();
                return true;
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }

        /// <summary>
        /// Restituisce il numero di messaggi contenuti nella lista per ricevere lo stato di notifica
        /// </summary>
        /// <returns>Il numero di messaggi contenuti nella lista dei messaggi per ricevere lo stato di notifica</returns>
        public int CountMessageToNotify()
        {
            try
            {
                //Numero messaggi nella lista
                return _MessageListToNotify.Count;
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }

        /// <summary>
        /// Restituisce il messaggio inserito nella lista dei messaggi per ricevere lo stato di notifica alla posizione indicata
        /// </summary>
        /// <param name="index">Indice del messaggio nella lista</param>
        /// <returns>Il messaggio inserito nella lista dei messaggi per ricevere lo stato di notifica alla posizione indicata</returns>
        public VolaMessaggio GetMessageToNotify(int index)
        {
            try
            {
                IEnumerator Lista = _MessageListToNotify.GetEnumerator(index, 1);
                Lista.MoveNext();
                return (VolaMessaggio)Lista.Current;
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }

        /// <summary>
        /// Comando Vola che permette di conoscere lo status di invio di tutti gli sms nel caso in cui l'account utilizzata abbia abilitata la funzionalit di invio con notifica.
        /// In caso contrario lo stato di notifica restituito si riferisce alla sola presa in consegna del messaggio lato gateway VolaSMS.
        /// Se il comando va a buon fine restituisce una lista contenente lo stato informativo di ricezione del messaggio.
        /// Questo tipo di comando permette di inviare al gateway pi richieste di notifica contemporaneamente.
        /// </summary>
        /// <returns>True in caso positivo, False altrimenti</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>                         
        public bool GetStatusNotify(){

            try
            {
                 
                VolaMessaggio NuovoMessaggio;
                VolaMsdn NuovoMsdn;
                string Risposta;

                //Verfico che la lista non sia vuota
                if (_MessageListToNotify.Count ==0)
                    return false;

                //Make the url request
                string urlPath = "CMD=10&UID=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) +
                                 "&PWD=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) +
                                 "&SERIAL=" + Vola.VolaSmsPlus.VolaConstant.SerialCode;
                if (_VapCode.Length > 0)
                    urlPath = urlPath + "&VCODE=" + _VapCode;
                
                urlPath = urlPath + "&QUERYDATA=";

                //Ciclo per i messaggi nella lista
                IEnumerator Lista = _MessageListToNotify.GetEnumerator();
                while (Lista.MoveNext())
                {                    
                    NuovoMessaggio = (VolaMessaggio) Lista.Current;

                    IEnumerator ListaMsdn = NuovoMessaggio.listaDestinatari.GetEnumerator();
                    ListaMsdn.MoveNext();

                    NuovoMsdn = (VolaMsdn)ListaMsdn.Current;
                    urlPath = urlPath +  NuovoMessaggio.orderid[0] + VolaConstant.SeparatorNotify1;
                    urlPath = urlPath + VolaUtility.UrlEncode(NuovoMsdn.gsm) + VolaConstant.SeparatorNotify2;                    
                }
                
                //Elimino l'ultimo due caratteri
                urlPath = urlPath.Substring(0, urlPath.Length - 1);

                //Invia richiesta               
                Risposta = VolaUtility.InvioDatiServer(urlPath,_EncryptSSL,_AutomaticProxy,_ProxyHost,_ProxyPort,_ProxyUsername,_ProxyPassword);
                
                //Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta);
                if (Risposta == "" || Risposta == "NULL")
                    return false;

                bool Find;
                string[] ListaMessaggi;
                string[] Messaggio;
                string[] Orderid;
                string[] ListaOrderid = Risposta.Split(VolaConstant.SeparatorResNotify1.ToCharArray());
                Lista = _MessageListToNotify.GetEnumerator();
                //Ciclio per orderId (ricevuti)
                for (int i = 0; i < ListaOrderid.Length; i++)
                {
                    Orderid = ListaOrderid[i].Split(VolaConstant.SeparatorResNotify2);
                    if (Orderid.Length == 2)
                    {
                        if (Orderid[1] != "" && Orderid[1] != "NULL")
                        {
                            Lista.Reset();
                            Find = true;
                            while (Lista.MoveNext() && Find)
                            {
                                NuovoMessaggio = (VolaMessaggio)Lista.Current;
                                if (NuovoMessaggio.orderid[0] == Orderid[0])
                                {
                                    ListaMessaggi = Orderid[1].Split(VolaConstant.SeparatorResNotify3);
                                    NuovoMessaggio.listaDestinatari.Clear();
                                    for (int count = 0; count < ListaMessaggi.Length; count++)
                                    {
                                        Messaggio = ListaMessaggi[count].Split(VolaConstant.SeparatorResNotify4);
                                        NuovoMsdn = new VolaMsdn();
                                        NuovoMsdn.datatimestatus = Messaggio[0] + " " + Messaggio[1];
                                        NuovoMsdn.gsm = Messaggio[2];
                                        switch (Messaggio[3])
                                        { 
                                            case "1":
                                                NuovoMsdn.status = VolaMsdnStatus.MSDN_SENT;
                                                break;
                                            case "2":
                                                NuovoMsdn.status = VolaMsdnStatus.MSDN_SMSC;
                                                break;
                                            case "3":
                                                NuovoMsdn.status = VolaMsdnStatus.MSDN_DELIVERED;
                                                break;
                                            case "4":
                                                NuovoMsdn.status = VolaMsdnStatus.MSDN_ERROPERATOR;

                                                switch (Messaggio[4])
                                                {
                                                    case "000":
                                                        NuovoMsdn.reasoncode = VolaMsdnReasonCode.REASONCODE_UNKNOW;
                                                        break;
                                                    case "101":
                                                    case "108":
                                                        NuovoMsdn.reasoncode = VolaMsdnReasonCode.REASONCODE_ERROR_DELIVERY;
                                                        break;
                                                    case "103":
                                                        NuovoMsdn.reasoncode = VolaMsdnReasonCode.REASONCODE_LOK_OPERATOR;
                                                        break;
                                                    case "111":
                                                        NuovoMsdn.reasoncode = VolaMsdnReasonCode.REASONCODE_LOK_USER;
                                                        break;
                                                    case "120":
                                                        NuovoMsdn.reasoncode = VolaMsdnReasonCode.REASONCODE_SIM_FULL;
                                                        break;
                                                    case "123":
                                                        NuovoMsdn.reasoncode = VolaMsdnReasonCode.REASONCODE_ERROR_OPERATOR;
                                                        break;
                                                    default:
                                                        NuovoMsdn.reasoncode = VolaMsdnReasonCode.REASONCODE_ERROR_DELIVERY;
                                                        break;
                                                }

                                                break;
                                            default:
                                                NuovoMsdn.status = VolaMsdnStatus.MSDN_SENT;
                                                break;
                                        }
                                        
                                        NuovoMessaggio.listaDestinatari.Add(NuovoMsdn);
                                    }
                                    Find = false;
                                }
                            }
                        }
                    }
                }                
                return true;
            
            }
            catch (VolaException ex)
            {
                throw new VolaException(ex.Messaggio, ex.Codice);
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }
                   
        #endregion

        #region Metodi per la ricezione dei messaggi e gestione chiavi di ricezione
        /// <summary>
        /// Comando Vola che controlla se vi sono nuovi messaggi messaggi sul server e restituisce il numero di nuovi messaggi ricevuti.         
        /// </summary>
        /// <returns>Il numero dei nuovi messaggi che ci sono sul server</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>                                        
        public int CheckIncomingMessages() {
            return CheckIncomingMessages("");
        }

        /// <summary>
        /// Comando Vola che controlla se vi sono nuovi messaggi messaggi sul server e restituisce il numero di nuovi messaggi ricevuti.         
        /// </summary>
        /// <param name="Key">La chiave di ricezione di cui si richiede il comando</param>
        /// <returns>Il numero dei nuovi messaggi che ci sono sul server</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>                                           
        public int CheckIncomingMessages(string Key){

            try
            {
                 //Make the url request
                string Risposta;

                string urlPath = "CMD=6&UID=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) +
                                 "&PWD=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) +
                                 "&SERIAL=" + Vola.VolaSmsPlus.VolaConstant.SerialCode;
                if (Key.Length > 0)
                    urlPath = urlPath + "&KEY=" + VolaUtility.UrlEncode(Key);
                 if (_VapCode.Length > 0)
                    urlPath = urlPath + "&VCODE=" + _VapCode;
                
                //Invia richiesta               
                Risposta = VolaUtility.InvioDatiServer(urlPath,_EncryptSSL,_AutomaticProxy,_ProxyHost,_ProxyPort,_ProxyUsername,_ProxyPassword);
                
                //Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta);

                if (VolaUtility.IsInt(Risposta))
                    return int.Parse(Risposta);
                else
                    return -1;
            
            }
            catch (VolaException ex)
            {
                throw new VolaException(ex.Messaggio, ex.Codice);
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }

        /// <summary>
        /// Comando Vola che restituisce la lista dei nuovi messaggi ricevuti dallaccount dallultima esecuzione del comando andata a buon fine.
        /// Se non vi sono nuovi messaggi la lista risulter vuota ed il comando restituir valore NULL.
        /// </summary>
        /// <returns>La lista dei nuovi messaggi ricevuti; NULL altrimenti</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>                                            
        public ArrayList GetIncomingMessages()
        {
            return GetIncomingMessages("");
        }
        
        /// <summary>
        /// Comando Vola che restituisce la lista dei nuovi messaggi ricevuti dallaccount dallultima esecuzione del comando andata a buon fine.
        /// La lista degli messaggi restituita dipende dal client applicativo che effettua la richiesta.
        /// La discriminazione viene fatta in base al valore del parametro KEY (chiave di ricezione) inviato nella richiesta.
        /// Se non vi sono nuovi messaggi la lista risulter vuota ed il comando restituir valore NULL.
        /// </summary>
        /// <param name="Key">La chiave di ricezione di cui si richiede il comando</param>
        /// <returns>La lista dei nuovi messaggi ricevuti; NULL altrimenti</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>                                                    
        public ArrayList GetIncomingMessages(string Key)
        {

            try
            {
                 //Make the url request
                string Risposta;

                string urlPath = "CMD=4&UID=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) +
                                 "&PWD=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) +
                                 "&SERIAL=" + Vola.VolaSmsPlus.VolaConstant.SerialCode;
                if (Key.Length > 0)
                    urlPath = urlPath + "&KEY=" + VolaUtility.UrlEncode(Key);
                 if (_VapCode.Length > 0)
                    urlPath = urlPath + "&VCODE=" + _VapCode;
                if (_TestMode)
                    urlPath = urlPath + "&TEST=1";
                else
                    urlPath = urlPath + "&TEST=0";
                
                //Invia richiesta               
                Risposta = VolaUtility.InvioDatiServer(urlPath,_EncryptSSL,_AutomaticProxy,_ProxyHost,_ProxyPort,_ProxyUsername,_ProxyPassword);
                
                //Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta);

                int pos = Risposta.IndexOf(' ');
                string num = Risposta.Substring(0, pos);
                string messaggi = Risposta.Substring(pos);

                if (VolaUtility.IsInt(num) != true)
                    return null;

                if (int.Parse(num) == 0 || messaggi.Length ==0 || messaggi == "NULL")
                    return null;

                ArrayList ListaRisposta = new ArrayList();
                VolaMessaggio MessaggioVola;
                string[] rispostaSplit;
                char[] charSeparators = new char[] { ' ' };
                rispostaSplit = Risposta.Split(charSeparators, 2);
                Risposta = rispostaSplit[1];

                string[] ListaMessaggi = Risposta.Split(VolaConstant.SeparatorRecive1); 
                //Ciclo sui messaggi ricevuti
                for (int i = 0; i < ListaMessaggi.Length; i++)
                {
                    string[] Messaggio = ListaMessaggi[i].Split(VolaConstant.SeparatorRecive2);
                    MessaggioVola = new VolaMessaggio();
                    MessaggioVola.dataora = Messaggio[0] + " " + Messaggio[1];
                    MessaggioVola.mittente = Messaggio[2];
                    MessaggioVola.testo = Messaggio[4];
                    MessaggioVola.key = Messaggio[3];

                    ListaRisposta.Add(MessaggioVola);
                }

                return ListaRisposta;
            
            }
            catch (VolaException ex)
            {
                throw new VolaException(ex.Messaggio, ex.Codice);
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }

        /// <summary>
        /// Comando Vola che conferma al server la corretta ricezione dei messggi SMS effettuata mediante lultima esecuzione del comando GetIncomingMessages.
        /// Questo comando deve essere eseguito dopo il comando GetIncomingMessages per confermare al gateway di VolaSMS l'avvenuta ricezione dei messaggi. 
        /// Se il comando non viene inviato, il gateway di VolaSMS restituir, alla ricezione del comando CMD=4, nuovamente l'elenco degli sms MO.
        /// </summary>
        /// <returns>True in caso positivo, False altrimenti</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>                                                            
        public bool DelIncomingMessages()
        {
            return DelIncomingMessages("");
        }

        /// <summary>
        /// Comando Vola che conferma al server la corretta ricezione dei messggi SMS effettuata mediante lultima esecuzione del comando GetIncomingMessages.
        /// Questo comando deve essere eseguito dopo il comando GetIncomingMessages per confermare al gateway di VolaSMS l'avvenuta ricezione dei messaggi. 
        /// Se il comando non viene inviato, il gateway di VolaSMS restituir, alla ricezione del comando CMD=4, nuovamente l'elenco degli sms MO.
        /// </summary>  
        /// <param name="Key">La chiave di ricezione di cui si richiede il comando</param>      
        /// <returns>True in caso positivo, False altrimenti</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>                                                                
        public bool DelIncomingMessages(string Key)
        {

            try
            {
                 //Make the url request
                string Risposta;

                string urlPath = "CMD=5&UID=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) +
                                 "&PWD=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) +
                                 "&SERIAL=" + Vola.VolaSmsPlus.VolaConstant.SerialCode;
                if (Key.Length > 0)
                    urlPath = urlPath + "&KEY=" + VolaUtility.UrlEncode(Key);

                 if (_VapCode.Length > 0)
                    urlPath = urlPath + "&VCODE=" + _VapCode;
                                
                //Invia richiesta               
                Risposta = VolaUtility.InvioDatiServer(urlPath,_EncryptSSL,_AutomaticProxy,_ProxyHost,_ProxyPort,_ProxyUsername,_ProxyPassword);
                
                //Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta);
               
                return true;
            
            }
            catch (VolaException ex)
            {
                throw new VolaException(ex.Messaggio, ex.Codice);
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }

        /// <summary>
        /// Comando Vola che restituisce il testo dell'autoresponder sulla chiave di ricezione
        /// </summary>  
        /// <param name="Key">La chiave di ricezione di cui si richiede il comando</param>      
        /// <returns>True in caso positivo, False altrimenti</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>                                                                
        public string GetAutoResponder(string Key){

            try
            {
                 //Make the url request
                string Risposta;
            
                string urlPath = "CMD=16&UID=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) +
                                 "&PWD=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) +
                                 "&SERIAL=" + Vola.VolaSmsPlus.VolaConstant.SerialCode +
                                 "&KEY=" + VolaUtility.UrlEncode(Key);;
              
                 if (_VapCode.Length > 0)
                    urlPath = urlPath + "&VCODE=" + _VapCode;
                                
                //Invia richiesta               
                Risposta = VolaUtility.InvioDatiServer(urlPath,_EncryptSSL,_AutomaticProxy,_ProxyHost,_ProxyPort,_ProxyUsername,_ProxyPassword);
                
                //Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta);

                if (Risposta == "AUTORESPONDER DISABLED!")
                    Risposta = "";
              
                return Risposta;
            
            }
            catch (VolaException ex)
            {
                throw new VolaException(ex.Messaggio, ex.Codice);
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }

        /// <summary>
        /// Comando Vola che imposta il testo dell'autoresponder sulla chiave di ricezione
        /// </summary>  
        /// <param name="Key">La chiave di ricezione di cui si richiede il comando</param>      
        /// <param name="TestoAutoResponder">Il testo del nuovo messaggio di Autoresponder</param>      
        /// <returns>True in caso positivo, False altrimenti</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>        
        public bool SetAutoResponder(string Key,string TestoAutoResponder){

            try
            {
                 //Make the url request
                string Risposta;
            
                string urlPath = "CMD=17&UID=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) +
                                 "&PWD=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) +
                                 "&SERIAL=" + Vola.VolaSmsPlus.VolaConstant.SerialCode +
                                 "&KEY=" + VolaUtility.UrlEncode(Key) +
                                 "&AUTORESPONDER=" + VolaUtility.UrlEncode(TestoAutoResponder);
              
                 if (_VapCode.Length > 0)
                    urlPath = urlPath + "&VCODE=" + _VapCode;
                                
                //Invia richiesta               
                Risposta = VolaUtility.InvioDatiServer(urlPath,_EncryptSSL,_AutomaticProxy,_ProxyHost,_ProxyPort,_ProxyUsername,_ProxyPassword);
                
                //Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta);

                return true;
            
            }
            catch (VolaException ex)
            {
                throw new VolaException(ex.Messaggio, ex.Codice);
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }
        #endregion

        #region Metodi e propriet per la gestione dei mittenti
        /// <summary>
        /// Lista dei mittenti associati all'account
        /// </summary>
        public ArrayList ListSender
        {
            get
            {
                return _ListSender;
            }
            set
            {
                _ListSender = value;
            }
        }

        /// <summary>
        /// Svuota la lista dei mittenti, attezione non elimina i mittenti associati all'account
        /// </summary>
        /// <returns>True in caso positivo, False altrimenti</returns>
        public bool ClearListSender()
        {
            try
            {
                //Cancella tutta la lista
                _ListSender.Clear();
                return true;
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }

        /// <summary>
        /// Restituisce il numero dei mittenti contenuti nella lista dei mittenti associati all'account
        /// </summary>
        /// <returns>Il numero di messaggi contenuti nella lista dei messaggi da inviare al gateway Vola</returns>
        public int CountListSender()
        {
            try
            {
                //Numero messaggi nella lista
                return _ListSender.Count;
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }
        
        /// <summary>
        /// Comando Vola che permette l'inserimento di un nuovo mittente alla lista dei mittenti dell'account Vola
        /// </summary>
        /// <param name="Sender">Il nuovo mittente da inserire; il campo pu contenere al max 11 caratteri alfanumerici
        /// Nel caso in cui il campo non sia correttamente valorizzato il gateway restituir un errore.</param>
        /// <returns>True in caso positivo, False altrimenti</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>                                        
        public bool AddSender(string Sender)
        {
            try
            {
                
                string Risposta;

                if (Sender.Length < 1)
                    return false;

                //Se  un cellulare lo imposto nel formato corretto
                string SenderOK = Sender;
                SenderOK = VolaCheckGsm.CheckGSMNumber(SenderOK, VolaCheckGsm.FormatGsm.EXISTCOUNTRY_MSDN);
                if (SenderOK.Length==0)
                    SenderOK = Sender;
           
                //Azzerro la lista di cui occorrer leggere nuovavemte dal gateway
                _ListSender.Clear();

                //Make the url request
                string urlPath = "CMD=30&UID=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) +
                                 "&PWD=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) +
                                 "&SERIAL=" + Vola.VolaSmsPlus.VolaConstant.SerialCode +
                                 "&SENDER=" + VolaUtility.UrlEncode(Sender);
            
                if (_VapCode.Length > 0)
                    urlPath = urlPath + "&VCODE=" + _VapCode;

                //Invia richiesta               
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _ProxyPassword);

                //Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta);
              
                return true;

            }
            catch (VolaException ex)
            {
                throw new VolaException(ex.Messaggio, ex.Codice);
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }

        /// <summary>
        /// Comando Vola che permette l'inserimento di un nuovo mittente alla lista dei mittenti dell'account Vola
        /// </summary>
        /// <param name="Sender">Il nuovo mittente da inserire; il campo pu contenere al max 14 caratteri numerici
        /// Nel caso in cui il campo non sia correttamente valorizzato il gateway restituir un errore.
        /// Il mittente dovr essere autenticato attraverso la tecnica del PINCODE. 
        /// Lesecuzione del comando genera linvio di un SMS contente un pincode sul numero di cellulare da autenticare.</param>
        /// <returns>True in caso positivo, False altrimenti</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>      
        public int AddSenderGsm(string Sender)
        {

            try
            {
                //Make the url request
                string Risposta;
                string SenderOK;

                if (Sender.Length < 1)
                    return -1;

                //Se non  un cellulare esco altrimenti lo imposto nel formato corretto
                SenderOK = Sender;
                SenderOK = VolaCheckGsm.CheckGSMNumber(SenderOK, VolaCheckGsm.FormatGsm.EXISTCOUNTRY_MSDN);
                if (SenderOK.Length == 0)
                    return -1;                
               
                //Azzerro la lista di cui occorrer leggere nuovavemte dal gateway
                _ListSender.Clear();

                string urlPath = "CMD=34&UID=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) +
                                 "&PWD=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) +
                                 "&SERIAL=" + Vola.VolaSmsPlus.VolaConstant.SerialCode +
                                 "&SENDER=" + VolaUtility.UrlEncode(SenderOK);

                if (_VapCode.Length > 0)
                    urlPath = urlPath + "&VCODE=" + _VapCode;

                //Invia richiesta               
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _ProxyPassword);

                //Verifica risposta
                if (Risposta == "42")
                    return 0;
                else
                    Risposta = VolaUtility.GestioneRisposta(Risposta);

                return 1;

            }
            catch (VolaException ex)
            {
                throw new VolaException(ex.Messaggio, ex.Codice);
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }

        /// <summary>
        /// Comando Vola che permette la validazione del pincode per lautenticazione di un mittente (numero di cellulare)
        /// </summary>
        /// <param name="Sender">Il nuovo mittente da autenticare.</param>
        /// <param name="Pincode">Il codice ricevuto tramite SMS sul numero di cellulare nonch il mittente da autenticare.</param>
        /// <returns>True in caso positivo, False altrimenti</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>      
        public bool AuthenticateSender(string Sender, string Pincode)
        {
            try
            {
                //Make the url request
                string Risposta;

                //Se non  un cellulare esco altrimenti lo imposto nel formato corretto
                string SenderOK = Sender;
                SenderOK = VolaCheckGsm.CheckGSMNumber(SenderOK, VolaCheckGsm.FormatGsm.EXISTCOUNTRY_MSDN);
                if (SenderOK.Length == 0)
                    return false;

                if (Pincode.Length < 1)
                    return false;

                //Azzerro la lista di cui occorrer leggere nuovavemte dal gateway
                _ListSender.Clear();

                string urlPath = "CMD=33&UID=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) +
                                 "&PWD=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) +
                                 "&SERIAL=" + Vola.VolaSmsPlus.VolaConstant.SerialCode +
                                 "&SENDER=" + VolaUtility.UrlEncode(SenderOK) +
                                 "&PINCODE=" + VolaUtility.UrlEncode(Pincode);

                if (_VapCode.Length > 0)
                    urlPath = urlPath + "&VCODE=" + _VapCode;

                //Invia richiesta               
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _ProxyPassword);

                //Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta);
               
                return true;

            }
            catch (VolaException ex)
            {
                throw new VolaException(ex.Messaggio, ex.Codice);
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }

        /// <summary>
        /// Comando Vola che permette di cancellare un mittente alla lista dei mittenti dell'account Vola
        /// </summary>
        /// <param name="Sender">Il mittente da cancellare.</param>
        /// <returns>True in caso positivo, False altrimenti</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>      
        public bool RemoveSender(string Sender)
        {
            try
            {
                //Make the url request
                string Risposta;

                //Se  un cellulare lo imposto nel formato corretto
                string SenderOK = Sender;
                SenderOK = VolaCheckGsm.CheckGSMNumber(SenderOK, VolaCheckGsm.FormatGsm.EXISTCOUNTRY_MSDN);
                if (SenderOK.Length == 0)
                    SenderOK = Sender;

                //Azzerro la lista di cui occorrer leggere nuovavemte dal gateway
                _ListSender.Clear();

                string urlPath = "CMD=31&UID=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) +
                                 "&PWD=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) +
                                 "&SERIAL=" + Vola.VolaSmsPlus.VolaConstant.SerialCode +
                                 "&SENDER=" + VolaUtility.UrlEncode(SenderOK);

                if (_VapCode.Length > 0)
                    urlPath = urlPath + "&VCODE=" + _VapCode;

                //Invia richiesta               
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _ProxyPassword);

                //Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta);

                return true;

            }
            catch (VolaException ex)
            {
                throw new VolaException(ex.Messaggio, ex.Codice);
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }

        /// <summary>
        /// Comando Vola che permette di inizializzre la lista dei mittenti Vola 
        /// </summary>       
        /// <returns>True in caso positivo, False altrimenti</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>             
        /// <exception cref="System.Exception">Generata in caso di errore non previsto</exception>      
        public bool GetListSender()
        {
            try
            {
                //Make the url request
                string PRlstSender;
                _ListSender.Clear();

                string urlPath = "CMD=32&UID=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) +
                                 "&PWD=" + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) +
                                 "&SERIAL=" + Vola.VolaSmsPlus.VolaConstant.SerialCode;
                               

                if (_VapCode.Length > 0)
                    urlPath = urlPath + "&VCODE=" + _VapCode;

                //Invia richiesta               
                PRlstSender = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _ProxyPassword);

                //Verifica risposta
                PRlstSender = VolaUtility.GestioneRisposta(PRlstSender);

                //Lista mittenti
                string[] DatiMitente;
                VolaSender NuovoMittente;
                string[] ListaMitt = PRlstSender.Split(VolaConstant.SeparatorSender1);
                for (int i = 0; i < ListaMitt.Length; i++){
                    DatiMitente = ListaMitt[i].Split(VolaConstant.SeparatorSender2);
                    NuovoMittente = new VolaSender();
                    NuovoMittente.mittente =DatiMitente[0];
                    switch (DatiMitente[1]){
                        case "P":
                            NuovoMittente.type = VolaTypeSender.PREDEFINITO;
                            break;
                        case "A":
                            NuovoMittente.type = VolaTypeSender.AUTENTICATO;
                            break;
                        case "V":
                            NuovoMittente.type = VolaTypeSender.VOLA;
                            break;
                        case "T":
                            NuovoMittente.type = VolaTypeSender.DA_AUTENTICARE;
                            break;
                        default:
                            NuovoMittente.type = VolaTypeSender.ERRATO;
                            break;

                    }                   
                    _ListSender.Add(NuovoMittente);
                }

                return true;

            }
            catch (VolaException ex)
            {
                throw new VolaException(ex.Messaggio, ex.Codice);
            }
            catch (Exception ex)
            {
                throw new VolaException("Errore generico Vola SMS: " + ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC);

            }
        }
        #endregion


    }

}