/*
 * Version: $RCSfile: VolaStructure.cs,v $ - $Revision: 1.3 $ - $Date: 2009/02/17 09:32:30 $
 *  
 *  This file is part of VolaSmsPlus.
 *
 *   VolaSmsPlus is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published by
 *   the Free Software Foundation, either version 3 of the License.
 * 
 *   VolaSmsPlus is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with VolaSmsPlus.  If not, see <http://www.gnu.org/licenses/>.
**/


using System;
using System.Text;
using System.Collections;

namespace Vola.VolaSmsPlus
{
    /// <summary>
    /// Elenco Errori Vola
    /// </summary>
    public enum VolaError
    {
        VOLASMSPLUS_OK = 1,
        VOLASMSPLUS_ERROR_AUTHENTICATION = 99,
        VOLASMSPLUS_ERROR_CGI = 98,
        VOLASMSPLUS_ERROR_CREDIT = 97,
        VOLASMSPLUS_ERROR_GATEWAY = 96,
        VOLASMSPLUS_ERROR_SENTSMS = 95,
        VOLASMSPLUS_ERROR_VAPCODE = 94,
        VOLASMSPLUS_ERROR_AUTORESPONDER = 93,
        VOLASMSPLUS_ERROR_WRONG = 88,
        VOLASMSPLUS_ERROR_COMMAND = 89,
        VOLASMSPLUS_ERROR_SENDER_LIMIT_REACHED = 40,
        VOLASMSPLUS_ERROR_SENDER_AUTH = 41,
        VOLASMSPLUS_ERROR_SENDER_PINCODE = 42,
        VOLASMSPLUS_ERROR_SENDER_PINCODE2 = 43,
        VOLASMSPLUS_ERROR_SENDER_ALREADY_EXIST = 44,
        VOLASMSPLUS_ERROR_SENDER_LIST_KO = 45,
        VOLASMSPLUS_ERROR_SENDER_BLOCK = 46,
        VOLASMSPLUS_ERROR_SENDER_INVALID = 47,
        VOLASMSPLUS_ERROR_NETWORK = 10,
        VOLASMSPLUS_ERROR_GENERIC = 11,
        VOLASMSPLUS_ERROR_PROXY = 13,
    }

    /// <summary>
    /// Informazioni utente Vola
    /// </summary>
    public struct VolaProfiloUtente
    {
        private string _profilo;
        private string _CreditoSms;
        private string _CreditoMms;
        private string _LicenzaVolaOffice;
        private string _MittentePredefinito;
        private bool _ServizioNotificheUtente;
        private bool _ServizioNotificheServer;
        private bool _ServizioEstero;
        private string[] _ChiaviRicezione;
        private string _MittentiVola;
        private string _MittentiDisabilitati;
        private string _DataScadenzaCredito;
        private string _ListaMittenti;

        /// <summary>
        /// Valore stringa che pu assume S = GOLD L = SILVER
        /// </summary>
        public string profilo
        {
            get { return _profilo;}
            set {_profilo = value;}
        }

        /// <summary>
        /// Valore stringa con il credito utile per l'invio di SMS
        /// </summary>
        public string CreditoSms
        {
            get { return _CreditoSms; }
            set { _CreditoSms = value; }
        }

        /// <summary>
        /// Valore stringa con il credito utile per l'invio di MMS
        /// </summary>
        public string CreditoMms
        {
            get { return _CreditoMms; }
            set { _CreditoMms = value; }
        }

        /// <summary>
        /// Lista licenze Vola For Office; in disuso
        /// </summary>
        public string LicenzaVolaOffice
        {
            get { return _LicenzaVolaOffice; }
            set { _LicenzaVolaOffice = value; }
        }

        /// <summary>
        /// Mittente predefinito Vola
        /// </summary>
        public string MittentePredefinito
        {
            get { return _MittentePredefinito; }
            set { _MittentePredefinito = value; }
        }

        /// <summary>
        /// Servizio di notifiche abilitato per l'invio di SMS dell'utente
        /// </summary>
        public bool ServizioNotificheUtente
        {
            get { return _ServizioNotificheUtente; }
            set { _ServizioNotificheUtente = value; }
        }

        /// <summary>
        /// Servizio di notifiche abilitato per il gateway; in disuso
        /// </summary>
        public bool ServizioNotificheServer
        {
            get { return _ServizioNotificheServer; }
            set { _ServizioNotificheServer = value; }
        }

        /// <summary>
        /// Servizio di notifice abilitato per l'invio di SMS dell'utente
        /// </summary>
        public bool ServizioEstero
        {
            get { return _ServizioEstero; }
            set { _ServizioEstero = value; }
        }

        /// <summary>
        /// Lista chiave di ricezioni attive dell'utente
        /// </summary>
        public string[] ChiaviRicezione
        {
            get { return _ChiaviRicezione; }
            set { _ChiaviRicezione = value; }
        }

        /// <summary>
        /// Lista dei mittenti Vola validi; in disuso
        /// </summary>
        public string MittentiVola
        {
            get { return _MittentiVola; }
            set { _MittentiVola = value; }
        }

        /// <summary>
        /// Lista dei mittenti Vola bloccati dl gateaway; in disuso
        /// </summary>
        public string MittentiDisabilitati
        {
            get { return _MittentiDisabilitati; }
            set { _MittentiDisabilitati = value; }
        }

        /// <summary>
        /// Data di scadenza del credito SMS e MMS Vola
        /// </summary>
        public string DataScadenzaCredito
        {
            get { return _DataScadenzaCredito; }
            set { _DataScadenzaCredito = value; }
        }

        /// <summary>
        /// Lista dei mittenti dell'utente; in disuso
        /// </summary>
        public string ListaMittenti
        {
            get { return _ListaMittenti; }
            set { _ListaMittenti = value; }
        }

    }

    /// <summary>
    /// Elenco dei possibili stati in cui si pu trovare un messaggio Vola
    /// </summary>
    public enum VolaMsdnStatus
    {
        MSDN_INLIST = 0,       
        MSDN_SENT = 1,
        MSDN_SMSC = 2,
        MSDN_DELIVERED = 3,
        MSDN_ERROPERATOR = 4,
        MSDN_ERRTEXT = -1,
        MSDN_ERRSENDER = -2,
        MSDN_ERRDATETIME = -3,
        MSDN_ERRPHONEGSM = -4,        
        MSDN_ERRGATEWAY = -5
    }

    /// <summary>
    /// Elenco delle possibili cause per cui un SMS non  stato recapito
    /// </summary>
    public enum VolaMsdnReasonCode
    {
        REASONCODE_OK = 1,
        REASONCODE_UNKNOW = 2, 
        REASONCODE_LOK_OPERATOR = 3,
        REASONCODE_ERROR_DELIVERY = 4,
        REASONCODE_LOK_USER = 5,
        REASONCODE_SIM_FULL = 6,
        REASONCODE_ERROR_OPERATOR = 7
    }

    /// <summary>
    /// Classe per un destinatario di un messaggio SMS
    /// </summary>
    public class VolaMsdn
    {
        private VolaMsdnStatus _status;
        private string _gsm;
        private VolaMsdnReasonCode _reasoncode;
        private string _datatimestatus;

        /// <summary>
        /// Costrutore
        /// </summary>
        public VolaMsdn()
        {
            _status = VolaMsdnStatus.MSDN_INLIST;
            _reasoncode = VolaMsdnReasonCode.REASONCODE_OK;
            _datatimestatus = "";
        }

        /// <summary>
        /// Costrutore
        /// </summary>
        /// <param name="gsm"></param>
        public VolaMsdn(string gsm)
        {
            _status = VolaMsdnStatus.MSDN_INLIST;
            _gsm = gsm;
        }

        /// <summary>
        /// Numero GSM
        /// </summary>
        public string gsm
        {
            get { return _gsm; }
            set { _gsm = value; }
        }

        /// <summary>
        /// Stato del messaggio
        /// </summary>
        public VolaMsdnStatus status
        {
            get { return _status; }
            set { _status = value; }
        }

        /// <summary>
        /// Possibile cause di un messaggio non recapitato; solo quando si richiede lo stato di notifica
        /// </summary>
        public VolaMsdnReasonCode reasoncode
        {
            get { return _reasoncode; }
            set { _reasoncode = value; }
        }

        /// <summary>
        /// Data e ora dello stato in cui si trova il messaggio; solo quando si richiede lo stato di notifica
        /// </summary>
        public string datatimestatus
        {
            get { return _datatimestatus; }
            set { _datatimestatus = value; }
        }


    }

    /// <summary>
    /// Classe messaggio SMS
    /// </summary>
    public class VolaMessaggio
    {
        private ArrayList _listaDestinatari;
        private string _mittente;
        private string _testo;
        private string _dataora;
        private int _cid;
        private string _key;
        private string[] _orderdId;

       /// <summary>
       /// Costruttore
       /// </summary>
        public VolaMessaggio()
        {
            _listaDestinatari = new ArrayList();
        }

        /// <summary>
        /// Lista dei destinatari
        /// </summary>
        public ArrayList listaDestinatari
        {
            get { return _listaDestinatari; }
            set { _listaDestinatari = value; }
        }

        /// <summary>
        /// Mittente del messaggio
        /// </summary>
        public string mittente
        {
            get { return _mittente; }
            set { _mittente = value; }
        }

        /// <summary>
        /// Chiave ricezione del messaggio
        /// </summary>
        public string key
        {
            get { return _key; }
            set { _key = value; }
        }

        /// <summary>
        /// Testo del messaggio
        /// </summary>
        public string testo
        {
            get { return _testo; }
            set { _testo = value; }
        }

        /// <summary>
        /// Data e ora del messaggio
        /// </summary>
        public string dataora
        {
            get { return _dataora; }
            set { _dataora = value; }
        }

        /// <summary>
        /// Numero di identificazione SMS prima dell'invio
        /// </summary>
        public int cid
        {
            get { return _cid; }
            set { _cid = value; }
        }
        /// <summary>
        /// Numero di identificazione SMS dopo invio SMS
        /// </summary>
        public string[] orderid
        {
            get { return _orderdId; }
            set { _orderdId = value; }
        }

    }


    /// <summary>
    /// Costanti Vola
    /// </summary>
    public struct VolaConstant
    {
        private const string LinkVola = "www.vola.it";
        private const string Gateway = "sms.vola.it";
        private const string LinkHome = "/index.php";
        private const string Support = "/supporto/index.php";
        private const string _VolaAddressGateway = "sms.vola.it/cgi/volasms_gw_plus2.php";
        private const string _SerialCode = "TR45GDLBO730HDUIEQJ5";
        private const string _HttpProtocol = "http://";
        private const string _HttpsProtocol = "https://";
        private const string _Methode = "POST";
        private const string _ContentType = "application/x-www-form-urlencoded";
        private const string _CaractOK = "[A-Za-z0-9\n !\"#$%&'()*+,-./:;<=>?@[\\]^_`{|}~ߧ]";
                                                                            
        private const string _CaractDouble = "[,\\,],^,{,|,},~,";
        private const string _SeparatorBlock = "\t";
        private const string _SeparatorPhone = ",";
        private const string _SeparatorGroup = "\r\n";
        private const string _SeparatorNotify1 = ":";
        private const string _SeparatorNotify2 = ";";
        private const char _SeparatorSender1 = ',';
        private const char _SeparatorSender2 = ':';
        private const char _SeparatorRespost1 = '\t';
        private const char _SeparatorRespost2 = ';';
        private const char _SeparatorRespost3 = ',';
        private const char _SeparatorProfilo1 = '\t';
        private const char _SeparatorProfilo2 = ',';        
        private const char _SeparatorRecive1 = '\x1f';
        private const char _SeparatorRecive2 = '\t';
        private const string _SeparatorResNotify1 = "\r\n";
        private const char _SeparatorResNotify2 = '\t';
        private const char _SeparatorResNotify3 = ';';
        private const char _SeparatorResNotify4 = ',';

        static public char SeparatorResNotify4
        {
            get { return _SeparatorResNotify4; }
        }
        static public char SeparatorResNotify3
        {
            get { return _SeparatorResNotify3; }
        }
        static public char SeparatorResNotify2
        {
            get { return _SeparatorResNotify2; }
        }
        static public string SeparatorResNotify1
        {
            get { return _SeparatorResNotify1; }
        }
        static public char SeparatorRecive1
        {
            get { return _SeparatorRecive1; }
        }
        static public char SeparatorRecive2
        {
            get { return _SeparatorRecive2; }
        }
        static public char SeparatorProfilo1
        {
            get { return _SeparatorProfilo1; }
        }
        static public char SeparatorProfilo2
        {
            get { return _SeparatorProfilo2; }
        }
        static public char SeparatorRespost1
        {
            get { return _SeparatorRespost1; }
        }
        static public char SeparatorRespost2
        {
            get { return _SeparatorRespost2; }
        }
        static public char SeparatorRespost3
        {
            get { return _SeparatorRespost3; }
        }
        static public char SeparatorSender2
        {
            get { return _SeparatorSender2; }
        }
        static public char SeparatorSender1
        {
            get { return _SeparatorSender1; }
        }
        static public string SeparatorNotify1
        {
            get { return _SeparatorNotify1; }
        }
        static public string SeparatorNotify2
        {
            get { return _SeparatorNotify2; }
        }
        static public string SeparatorGroup
        {
            get { return _SeparatorGroup; }
        }
        static public string SeparatorPhone
        {
            get { return _SeparatorPhone; }
        }
        static public string SeparatorBlock
        {
            get { return _SeparatorBlock; }
        }
        static public string CaractTextOK
        {
            get { return _CaractOK; }
        }
        static public string CaractDouble
        {
            get { return _CaractDouble; }
        }
        static public string Methode
        {
            get { return _Methode; }
        }
        static public string ContentType
        {
            get { return _ContentType; }
        }
        static public string HttpProtocol
        {
            get { return _HttpProtocol; }
        }
        static public string HttpVolaAddressGateway
        {
            get { return HttpProtocol + _VolaAddressGateway; }
        }
        static public string HttpsVolaAddressGateway
        {
            get { return _HttpsProtocol + _VolaAddressGateway; }
        }
        static public string SerialCode
        {
            get { return _SerialCode; }
        }

        static public string LinkHomePage
        {
            get { return _HttpsProtocol + LinkVola; }
        }

        static public string LinkSupport
        {
            get { return _HttpsProtocol + LinkVola + Support; }
        }

    }

    /// <summary>
    /// Eccezione che pu essere generata dalla classe Volasmsplus durante l'invio dei messaggi
    /// </summary>
    public class VolaException : ApplicationException
    {
        private string _messaggio;
        private VolaError _codice;

        /// <summary>
        /// Costruttore
        /// </summary>
        /// <param name="messaggio">Valore stringa descrizione dell'errore</param>
        /// <param name="codice">Valore intero codice dell'errore</param>
        public VolaException(string messaggio, VolaError codice)
            : base(messaggio)
        {
            _messaggio = messaggio;
            _codice = codice;
        }

        /// <summary>
        /// Descrizione dell'errore
        /// </summary>
        public string Messaggio
        {
            get
            {
                return _messaggio;
            }
        }

        /// <summary>
        /// Codice dell'errore
        /// </summary>
        public VolaError Codice
        {
            get
            {
                return _codice;
            }
        }
    }

    /// <summary>
    /// Classe mittente dei messaggi
    /// </summary>
    public struct VolaSender
    {
        private string _mittente;
        private VolaTypeSender _type;

        /// <summary>
        /// Mittente
        /// </summary>
        public string mittente
        {
            get { return _mittente; }
            set { _mittente = value; }
        }

        /// <summary>
        /// Tipo mittente
        /// </summary>
        public VolaTypeSender type
        {
            get { return _type; }
            set { _type = value; }
        }
    }

    /// <summary>
    /// Elenco tipi mittenti
    /// </summary>
    public enum VolaTypeSender
    {
        PREDEFINITO = 'P',
        VOLA= 'V',
        AUTENTICATO = 'A',
        DA_AUTENTICARE = 'T',
        ERRATO ='E'
    }
}