/*
 * Version: $RCSfile: VolaUtility.cs,v $ - $Revision: 1.3 $ - $Date: 2009/02/17 09:32:30 $
 *  
 *  This file is part of VolaSmsPlus.
 *
 *   VolaSmsPlus is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published by
 *   the Free Software Foundation, either version 3 of the License.
 * 
 *   VolaSmsPlus is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with VolaSmsPlus.  If not, see <http://www.gnu.org/licenses/>.
**/


using System;
using System.Net;
using System.Text;
using System.Web;
using System.IO;
using System.Security.Cryptography;

namespace Vola.VolaSmsPlus
{
    static internal class VolaUtility
    {
        /// <summary>
        /// Consente di trasformare il valore stringa di input e una stringa MD5 in output
        /// </summary>
        /// <param name="original">Valore stringa da trasformare</param>
        /// <returns>Valore stringa trasformata in codice MD5</returns>
        /// <exception cref="System.Exception">Generata se il valore stringa di input non  in un formato valido</exception>
        static internal string MD5Encode(string original)
        {
            try
            {
                Byte[] originalBytes;
                Byte[] encodedBytes;
                MD5 md5;

                md5 = new MD5CryptoServiceProvider();
                originalBytes = ASCIIEncoding.Default.GetBytes(original);
                encodedBytes = md5.ComputeHash(originalBytes);

                //Convert encoded bytes back to a 'readable' string
                return BitConverter.ToString(encodedBytes).Replace("-", "");
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        /// <summary>
        /// Verifica se il valore stringa di input  un numero e restituisce
        /// </summary>
        /// <param name="valore">Valore stringa da verificare</param>
        /// <returns>Ritorna True in caso positivo, False altrimenti</returns>
        /// <exception cref="System.Exception">Generata se il valore stringa di input non  in un formato valido</exception>
        static internal bool IsInt(string valore)
        {
            try
            {
                int.Parse(valore);
                return true;
            }
            catch
            {
                return false;
            }
        }

        /// <summary>
        /// Metodo per inviare i comandi al gateway e per ricevere e gestire la risposta
        /// </summary>
        /// <param name="Dati">Valore stringa contente il comando nella sintassi Vola</param>
        /// <param name="EncryptSSL">Valore booleano per il servizio di connessione sicura</param>
        /// <param name="AutomaticProxy">Valore booleano per il servizio di proxy automatico</param>
        /// <param name="ProxyHost">Valore stringa con il nome o l'indirizzo dell'host; solo in caso di AutomaticProxy non abilitato</param>
        /// <param name="ProxyPort">Valore stringa con il numero di porta dell'host; solo in caso di AutomaticProxy non abilitato</param>
        /// <param name="ProxyUsername">Valore stringa con la username dell'host; solo in caso di AutomaticProxy non abilitato</param>
        /// <param name="ProxyPassword">Valore stringa con la password dell'host; solo in caso di AutomaticProxy non abilitato</param>
        /// <returns>Valore stringa contente la risposta del Gateway Vola al comando inviato</returns>
        /// <exception cref="System.UriFormatException">Generata se l'indirizzo gateway non  pi valido</exception>
        /// <exception cref="SSystem.Net.WebException">Generata se si verifica un errore di connessione</exception>
        /// <exception cref="System.Exception">Generata se i valori di input non sono in un formato valido</exception>
        static internal string InvioDatiServer(string Dati, bool EncryptSSL, bool AutomaticProxy, string ProxyHost, string ProxyPort, string ProxyUsername, string ProxyPassword)
        {
            string gatewayUrl;
            HttpWebResponse resp = null;
            string Risposta;
            WebProxy Proxy = null;

            //Tipo d connessione
            if (EncryptSSL)
                gatewayUrl = Vola.VolaSmsPlus.VolaConstant.HttpsVolaAddressGateway;
            else
                gatewayUrl = Vola.VolaSmsPlus.VolaConstant.HttpVolaAddressGateway;

            try
            {
                //Crea oggetto per connessione
                HttpWebRequest request = (HttpWebRequest)WebRequest.Create(gatewayUrl);
                
                //Gestione proxy
                if (AutomaticProxy)
                    request.Proxy = null;
                else
                {
                    Uri newUri = new Uri(Vola.VolaSmsPlus.VolaConstant.HttpProtocol + ProxyHost + ":" + ProxyPort);
                    Proxy = new WebProxy();

                    if (ProxyUsername.Length > 0)
                        Proxy.Credentials = new NetworkCredential(ProxyUsername, ProxyPassword);
                    Proxy.Address = newUri;
                    request.Proxy = Proxy;
                }

                //Trasformo i dati stringa in byte
                byte[] postDataBytes = Encoding.UTF8.GetBytes(Dati);
                request.Method = Vola.VolaSmsPlus.VolaConstant.Methode;
                request.ContentType = Vola.VolaSmsPlus.VolaConstant.ContentType;
                request.ContentLength = postDataBytes.Length;
                request.Timeout = 30000;
                request.KeepAlive = true;
                
                Stream requestStream = request.GetRequestStream();
                requestStream.Write(postDataBytes, 0, postDataBytes.Length);
                requestStream.Close();
                
                // get response and write to console
                resp = (HttpWebResponse)request.GetResponse();
                StreamReader responseReader = new StreamReader(resp.GetResponseStream(), Encoding.UTF8);
                Risposta = responseReader.ReadToEnd();
              
                if (Risposta.Length == 0)
                {
                    return (((int)VolaError.VOLASMSPLUS_ERROR_NETWORK) + " Messaggio vuoto");
                }
                resp.Close();
                Risposta = Risposta.Substring(16, Risposta.Length - 36);                
                return Risposta;
            }
            catch (System.Net.WebException e)
            {
                return (((int)VolaError.VOLASMSPLUS_ERROR_NETWORK) + " " + e.Message);
            }
            catch (System.UriFormatException e)
            {
                return (((int)VolaError.VOLASMSPLUS_ERROR_PROXY) + " " + e.Message);
            }
            catch (Exception e)
            {
                return (((int)VolaError.VOLASMSPLUS_ERROR_NETWORK) + " " + e.Message);
            }
            finally
            {
                if (resp != null)
                {
                    resp.Close();
                }
            }
        }

        /// <summary>
        /// Gestore di Errore di VolaAddress.ocx 
        /// </summary>
        /// <param name="Risposta">Valore stringa contente la risposta dal gateway Vola comprensivo del codice di risposta</param>      
        /// <returns>Valore stringa con i dati di risposta dal gateway Vola senza il codice di risposta</returns>
        /// <exception cref="Vola.VolaSmsPlus.VolaException">Generata se la risposta del gateway conteneva un codice di risposta diverso da 0</exception>       
        static internal string GestioneRisposta(string Risposta)
        {
            VolaError Codice = 0;
            string Dati = "";
            string CodiceRisposta="";
            string Messaggio = "";

            //
            CodiceRisposta = Risposta.Substring(0, 2);
            if (IsInt(CodiceRisposta))
                Codice = (VolaError)Convert.ToInt32(CodiceRisposta);
            else
                Codice = VolaError.VOLASMSPLUS_ERROR_CGI;
            
            switch (Codice)
            {
                case VolaError.VOLASMSPLUS_ERROR_GENERIC:
                    //generic error 
                    Messaggio = "Errore generico nel funzionamento di VolaSmsPlus. Contattare il supporto tecnico\r\n";
                    break;

                case VolaError.VOLASMSPLUS_ERROR_NETWORK:
                    //Network Error 
                    Messaggio = "Si  verificato un problema di connessione ad Internet.\r\n" +
                    "Chiudere e riavviare l'applicazione. Se il problema persiste, controllare:\r\n" +
                    "\t- la connessione ad Internet\r\n;" +
                    "\t- le impostazioni del Proxy Server (se esistente) dal Menu File|Opzioni..";
                    break;

                case VolaError.VOLASMSPLUS_ERROR_AUTHENTICATION:
                    //Authorization error 
                    Messaggio = "Username o Password inserita errata. Riprovare.\r\n" +
                    "Si ricorda che i campi sono case sensitive, di conseguenza sia il nome utente che la password devono essere immessi con le lettere minuscole o maiuscole come sono state inserite in origine.";
                                      
                    break;

                case VolaError.VOLASMSPLUS_ERROR_CGI:
                    //CGI Error 
                    Messaggio = "Errore CGI. Sono stati passati parametri errati. Contattare il supporto tecnico.";                   
                    break;

                case VolaError.VOLASMSPLUS_ERROR_AUTORESPONDER:
                    //Autoresponder error
                    Messaggio = "Errore nel testo dell'autoresponder o chiave di ricezione scaduta.";
                    break;

                case VolaError.VOLASMSPLUS_ERROR_CREDIT:
                    //Credit Error 
                    Messaggio = "Credito insufficiente. E' necessario acquistare una ricarica.";
                    break;

                case VolaError.VOLASMSPLUS_ERROR_COMMAND:
                    //Command Error 
                    Messaggio = "Errore di comunicazione col Gateway SMS; i parametri ricevuti hanno provocato un errore.";
                    break;

                case VolaError.VOLASMSPLUS_ERROR_GATEWAY:
                    //Gateway Error 
                    Messaggio =  "Errore grave del gateway di Vola. Contattare il supporto tecnico!";                    
                    break;

                case VolaError.VOLASMSPLUS_ERROR_VAPCODE:
                    //Wrong VAPCode Error 
                    Messaggio = "L'applicazione contiene un VAPCODE errato. Contattare il supporto tecnico.";
                    
                    break;

                case VolaError.VOLASMSPLUS_ERROR_SENTSMS:
                    Messaggio = "Si  tentato di inviare uno o pi messaggi uguali, sar inviato un solo messaggio di quelli uguali.";
                    break;

                case VolaError.VOLASMSPLUS_ERROR_WRONG:
                    //General Error Gateway
                    Messaggio ="I dati inseriti non sono corretti per l'operazione da effettuare.";
                    break;               

                case VolaError.VOLASMSPLUS_ERROR_SENDER_LIMIT_REACHED: 
                    //Sender Error
                    Messaggio = "Impossibile aggiungere nuovi mittenti poich si  raggiunto il limite massimo del numero dei mittenti impostabili per ogni singolo account. Per aggiungere un nuovo mittente  necessario prima cancellarne uno gi presente nella lista dei mittenti disponibili.";
                    break;

                case VolaError.VOLASMSPLUS_ERROR_SENDER_AUTH:
                    //Sender Error
                    Messaggio = "Impossibile aggiungere il mittente poich il numero inserito richiede l'autenticazione con il codice di sblocco."; 
                    break;
              
                case VolaError.VOLASMSPLUS_ERROR_SENDER_PINCODE:
                    //Sender Error
                    Messaggio = "Il mittente che si  cercato di inserire non  stato aggiunto alla lista poich non  corretto.";                    
                    break;

                case VolaError.VOLASMSPLUS_ERROR_SENDER_PINCODE2:
                    //Sender Error
                    Messaggio = "Si  verificato un problema per cui non  stato possibile inviare il pincode sul numero di cellulare da autenticare.";
                    break;

                case VolaError.VOLASMSPLUS_ERROR_SENDER_LIST_KO:
                    //Sender Error
                    Messaggio = "Il mittente richiesto esiste gi nella lista dei mittenti impostati per laccount.";
                    break;

                case VolaError.VOLASMSPLUS_ERROR_SENDER_BLOCK:
                    //Sender Error
                    Messaggio = "Il mittente richiesto non ha una lunghezza valida (11 caratteri alfanumerici oppure 16 numerici)";
                    break;

                case VolaError.VOLASMSPLUS_ERROR_SENDER_INVALID:
                    //Sender Error
                    Messaggio = "Non  acconsentito aggiungere il mittente richiesto.";
                    break;
                      
                case VolaError.VOLASMSPLUS_ERROR_PROXY:
                    //Proxy Error
                    Messaggio = "ATTENZIONE, le impostazioni proxy sono errate, se non si conoscono informarmarsi presso il proprio amministratore di rete.";
                    break;
               
                case VolaError.VOLASMSPLUS_OK:
                    Messaggio = "";
                    if (Risposta.Length <= 2)
                        Dati = "";
                    else
                        Dati = Risposta.Substring(3);                    
                    break;
                default:                    
                    Messaggio = "Errore grave di comunicazione col server contattare il supporto tecnico.";
                    break;
            }

            if (Messaggio.Length > 0)
            {             
                throw new Vola.VolaSmsPlus.VolaException(Messaggio,Codice);
            }
            return Dati;
        }


        static internal String UrlEncode(String text)
        {
            // we do not want + for spaces but %20
            String enc = text.Replace(" ", "(fb_space)");
            enc = enc.Replace("", "(fb_euro)");
            enc = HttpUtility.UrlEncode(enc, System.Text.Encoding.Unicode);
            enc = enc.Replace("%00", "");
            enc = GetUpEncodedString(enc);
            enc = enc.Replace("(fb_space)", "%20");        
            enc = enc.Replace("(fb_euro)", "%80");
            return enc;

            
        }

        static internal string GetUpEncodedString(string strVal)
        {
            int intPos = 0;
            string strRet = "";

            while (intPos != -1 && strVal != "")
            {
                intPos = strVal.IndexOf("%", intPos + 1);
                if (intPos != -1 && intPos <= strVal.Length - 2)
                    strRet += strVal.Substring(strRet.Length, intPos - strRet.Length) + strVal.Substring(intPos, 3).ToUpper();
            }
            if (strRet == "")
                strRet = strVal;
            else
                strRet += strVal.Substring(strRet.Length);

            return strRet;
        }

        
    }
}