'
'  Version: $RCSfile: VolaSmsPlus.vb,v $ - $Revision: 1.2 $ - $Date: 2009/02/17 09:34:57 $
'
'  This file is part of VolaSmsPlus.
'
'   VolaSmsPlus is free software: you can redistribute it and/or modify
'   it under the terms of the GNU Lesser General Public License as published by
'   the Free Software Foundation, either version 3 of the License.
' 
'   VolaSmsPlus is distributed in the hope that it will be useful,
'   but WITHOUT ANY WARRANTY; without even the implied warranty of
'   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'   GNU Lesser General Public License for more details.
'
'   You should have received a copy of the GNU Lesser General Public License
'   along with VolaSmsPlus.  If not, see <http://www.gnu.org/licenses/>.
'

Imports System
Imports System.Text
Imports System.Web
Imports System.Reflection
Imports System.Text.RegularExpressions
Imports System.Collections

Namespace Vola.VolaSmsPlus
    ''' <summary>
    '''
    ''' </summary>
    Public Class VolaSmsPlus
        Private Const MAX_DIM_ARRAY As Integer = 1000
        Private Const MAX_DIM_ARRAY_SENDER As Integer = 30

        Private _UserName As String
        Private _Password As String
        Private _Notify As Boolean
        Private _LongMessage As Boolean
        Private _EncryptSSL As Boolean
        Private _AutomaticProxy As Boolean
        Private _ProxyHost As String
        Private _ProxyPort As String
        Private _ProxyUsername As String
        Private _ProxyPassword As String
        Private _VapCode As String

        Private _MessageListToSend As ArrayList
        Private _MessageListToNotify As ArrayList
        Private _ListSender As ArrayList

        Private _TestMode As Boolean
        Private _DefaultCountry As String

        ''' <summary>
        ''' Costruttore
        ''' </summary>
        Public Sub New()
            _TestMode = False
            _MessageListToSend = New ArrayList(MAX_DIM_ARRAY)
            _MessageListToNotify = New ArrayList(MAX_DIM_ARRAY)
            _ListSender = New ArrayList(MAX_DIM_ARRAY_SENDER)
            _DefaultCountry = "+39"
        End Sub

#Region "propriet per la gestione di VolaSmsPlus"
        ''' <summary>
        ''' Username dell'account Vola
        ''' </summary>
        Public Property UserName() As String
            Get
                Return _UserName
            End Get
            Set(ByVal value As String)
                _UserName = value
            End Set
        End Property
        ''' <summary>
        ''' Password dell'account Vola
        ''' </summary>
        Public Property Password() As String
            Get
                Return _Password
            End Get
            Set(ByVal value As String)

                _Password = value
            End Set
        End Property

        ''' <summary>
        ''' Abilita/Disabilita il servizio di notifica SMS per il prossimo invio
        ''' </summary>
        Public Property Notify() As Boolean
            Get
                Return _Notify
            End Get
            Set(ByVal value As Boolean)
                _Notify = value
            End Set
        End Property
        ''' <summary>
        ''' Abilita/Disabilita il servizio per poter inviare messaggi lunghi
        ''' </summary>
        Public Property LongMessage() As Boolean
            Get
                Return _LongMessage
            End Get
            Set(ByVal value As Boolean)
                _LongMessage = value
            End Set
        End Property
        ''' <summary>
        ''' Abilita/Disabilita utilizzo di un server proxy per la connessione al gateway Vola
        ''' </summary>
        Public Property AutomaticProxy() As Boolean
            Get
                Return _AutomaticProxy
            End Get
            Set(ByVal value As Boolean)
                _AutomaticProxy = value
            End Set
        End Property
        ''' <summary>
        ''' Abilita/Disabilita la connessione sicura al gateway Vola
        ''' </summary>
        Public Property EncryptSSL() As Boolean
            Get
                Return _EncryptSSL
            End Get
            Set(ByVal value As Boolean)
                _EncryptSSL = value
            End Set
        End Property
        ''' <summary>
        ''' Nome o Indirizzo dell'host per la connessione internet
        ''' </summary>
        Public Property ProxyHost() As String
            Get
                Return _ProxyHost
            End Get
            Set(ByVal value As String)
                _ProxyHost = value
            End Set
        End Property
        ''' <summary>
        ''' Porta dell'host per la connessione internet
        ''' </summary>
        Public Property ProxyPort() As String
            Get
                Return _ProxyPort
            End Get
            Set(ByVal value As String)
                _ProxyPort = value
            End Set
        End Property
        ''' <summary>
        ''' Eventuale Username dell'host per la connessione internet
        ''' </summary>
        Public Property ProxyUsername() As String
            Get
                Return _ProxyUsername
            End Get
            Set(ByVal value As String)
                _ProxyUsername = value
            End Set
        End Property
        ''' <summary>
        ''' Eventuale Password dell'host per la connessione internet
        ''' </summary>
        Public Property ProxyPassword() As String
            Get
                Return _ProxyPassword
            End Get
            Set(ByVal value As String)
                _ProxyPassword = value
            End Set
        End Property
        ''' <summary>
        ''' Abilita/Disabilita la funzionalit test per invio SMS
        ''' </summary>
        Public Property TestMode() As Boolean
            Get
                Return _TestMode
            End Get
            Set(ByVal value As Boolean)
                _TestMode = value
            End Set
        End Property
        ''' <summary>
        ''' Questo parametro deve essere utilizzato per chi ha aderito al programma VAP (Vola Affiliate Program).
        ''' Deve essere valorizzato con il codice alfanumerico (VAP CODE) assegnato nel momento della registrazione.
        ''' Per maggiori informazioni inviare una mail a supporto@vola.it
        ''' </summary>
        Public Property VapCode() As String
            Get
                Return _VapCode
            End Get
            Set(ByVal value As String)
                If value.Length > 10 OrElse value.Contains(" ") Then
                    Throw New VolaException("Il codice VAPCODE  errato.", VolaError.VOLASMSPLUS_ERROR_VAPCODE)
                End If
                _VapCode = value.Trim()
            End Set
        End Property
        ''' <summary>
        ''' Prefisso internazionale di default
        ''' </summary>
        Public Property DefaultCountry() As String
            Get
                Return _DefaultCountry
            End Get
            Set(ByVal value As String)
                _DefaultCountry = value
            End Set
        End Property
#End Region


#Region "Metodi per la gestione del profilo Vola"
        ''' <summary>
        ''' Comando Vola restituisce le informazioni sull'account Vola.
        ''' </summary>
        ''' <returns>Ritorna oggetto VolaProfiloUtente inizializzato con le informazioni sull'account Vola</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function GetProfile() As VolaProfiloUtente
            Dim Risposta As String = ""

            Try

                'Make the url request
                Dim urlPath As String = ((("CMD=18&UID=" & VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) & "&PWD=") + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) & "&SERIAL=") + Vola.VolaSmsPlus.VolaConstant.SerialCode & "&VCODE=") + _VapCode

                'Invia richiesta
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _
                _ProxyPassword)

                'Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta)

                'Parse Profile
                Dim ProfiloSplit As String() = Risposta.Split(VolaConstant.SeparatorProfilo1)
                Dim Profilo As New VolaProfiloUtente()

                'Riempo la struttura dati
                Profilo.profilo = ProfiloSplit(0)
                Profilo.CreditoSms = ProfiloSplit(1)
                Profilo.CreditoMms = ProfiloSplit(2)
                Profilo.LicenzaVolaOffice = ProfiloSplit(3)
                Profilo.MittentePredefinito = ProfiloSplit(4)
                If ProfiloSplit(5) = "S" Then
                    Profilo.ServizioNotificheUtente = True
                Else
                    Profilo.ServizioNotificheUtente = False
                End If
                If ProfiloSplit(6) = "S" Then
                    Profilo.ServizioNotificheServer = True
                Else
                    Profilo.ServizioNotificheServer = False
                End If
                If ProfiloSplit(7) = "S" Then
                    Profilo.ServizioEstero = True
                Else
                    Profilo.ServizioEstero = False
                End If

                If ProfiloSplit(8).Length > 0 AndAlso ProfiloSplit(8).ToUpper() <> "null".ToUpper() Then
                    Profilo.ChiaviRicezione = ProfiloSplit(8).Split(VolaConstant.SeparatorProfilo2)
                Else
                    Profilo.ChiaviRicezione = New String(-1) {}
                End If
                Profilo.MittentiVola = ProfiloSplit(9)
                Profilo.MittentiDisabilitati = ProfiloSplit(10)
                Profilo.DataScadenzaCredito = ProfiloSplit(11)
                Profilo.ListaMittenti = ProfiloSplit(12)
                Return Profilo
            Catch ex As VolaException
                Throw New VolaException(ex.Messaggio, ex.Codice)
            Catch ex As Exception
                Throw New VolaException("Errore grave durante la connessione al servizio SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)
            End Try
        End Function

        ''' <summary>
        ''' Comando Vola restituisce il numero credito residuo di VolaSMS. Il credito e' espresso in numero di messaggi.
        ''' </summary>
        ''' <returns>Ritorna un valore stringa con il credito residuo dell'account Vola</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function GetCreditSMS() As String
            Dim Risposta As String = ""

            Try

                'Make the url request
                Dim urlPath As String = (("CMD=1&UID=" & VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) & "&PWD=") + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) & "&SERIAL=") + Vola.VolaSmsPlus.VolaConstant.SerialCode
                If _VapCode.Length > 0 Then
                    urlPath = (urlPath & "&VCODE=") + _VapCode
                End If

                'Invia richiesta
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _
                _ProxyPassword)

                'Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta)

                Return Risposta
            Catch ex As VolaException
                Throw New VolaException(ex.Messaggio, ex.Codice)
            Catch ex As Exception
                Throw New VolaException("Errore grave durante la connessione al servizio SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function
#End Region

#Region "Metodi e propriet per InvioSMS"

        ''' <summary>
        ''' Lista dei messaggi da inviare o inviati al gateway Vola
        ''' </summary>
        Public Property MessageListToSend() As ArrayList
            Get
                Return _MessageListToSend
            End Get
            Set(ByVal value As ArrayList)
                _MessageListToSend = value
            End Set
        End Property

        ''' <summary>
        ''' Inserisce un nuovo messaggio nella lista dei messaggi da inviare al gateway Vola
        ''' </summary>
        ''' <param name="Gsm">Destinatario del messaggio (Numero Gsm)</param>
        ''' <param name="Testo">Testo del messaggio e NON pu contenere i caratteti 'tab' e 'cr_lf'.
        ''' Se il testo del messaggio supera i 160 caratteri il gateway di VolaSMS effettua uno split del testo
        ''' in modo che ad ogni destinatario vengano inviati N messaggi ciascuno lungo al massimo 160 caratteri.
        ''' Il numero massimo di caratteri che pu contenere il campo messaggio e' 1440 caratteri.</param>
        ''' <param name="Mittente">Mittente del gruppo di invio. E' il mittente con cui verra' inviato l'sms al destinatario o ai destinatari del messaggio.
        ''' Il campo avr effetto solo se l'account utilizzata supporta l'invio di messaggi con campo mittente modificabile.
        ''' In quest'ultimo caso il gateway di VolaSMS utilizzer il valore di default vola.it o SMS, nel caso in cui il profilo di listino impostato lo supporti.
        ''' Il campo pu contenere al max 11 caratteri alfanumerici, oppure 14 caratteri numerici.
        ''' Nel caso in cui il campo non sia correttamente valorizzato il gateway restituir un errore.</param>
        ''' <returns>Il messaggio inserito nella lista dei messaggi da inviare al gateway Vola</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function AddMessageToSend(ByVal Gsm As String, ByVal Testo As String, ByVal Mittente As String) As VolaMessaggio
            Return AddMessageToSend(Gsm, Testo, Mittente, DateTime.MinValue)
        End Function

        ''' <summary>
        ''' Inserisce un nuovo messaggio nella lista dei messaggi da inviare al gateway Vola
        ''' </summary>
        ''' <param name="Gsm">Destinatario del messaggio (Numero Gsm)</param>
        ''' <param name="Testo">Testo del messaggio e NON pu contenere i caratteti 'tab' e 'cr_lf'.
        ''' Se il testo del messaggio supera i 160 caratteri il gateway di VolaSMS effettua uno split del testo
        ''' in modo che ad ogni destinatario vengano inviati N messaggi ciascuno lungo al massimo 160 caratteri.
        ''' Il numero massimo di caratteri che pu contenere il campo messaggio e' 1440 caratteri.</param>
        ''' <param name="Mittente">Mittente del gruppo di invio. E' il mittente con cui verra' inviato l'sms al destinatario o ai destinatari del messaggio.
        ''' Il campo avr effetto solo se l'account utilizzata supporta l'invio di messaggi con campo mittente modificabile.
        ''' In quest'ultimo caso il gateway di VolaSMS utilizzer il valore di default vola.it o SMS, nel caso in cui il profilo di listino impostato lo supporti.
        ''' Il campo pu contenere al max 11 caratteri alfanumerici, oppure 14 caratteri numerici.
        ''' Nel caso in cui il campo non sia correttamente valorizzato il gateway restituir un errore.</param>
        ''' <param name="DataOra">Data e ora in cui il messaggio deve essere inviato</param>
        ''' <returns>Il messaggio inserito nella lista dei messaggi da inviare al gateway Vola</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function AddMessageToSend(ByVal Gsm As String, ByVal Testo As String, ByVal Mittente As String, ByVal DataOra As DateTime) As VolaMessaggio
            Dim listaGsm As String() = New String(0) {}
            listaGsm(0) = Gsm
            Return AddMessageToSend(listaGsm, Testo, Mittente, DataOra)

        End Function

        ''' <summary>
        ''' Inserisce un nuovo messaggio nella lista dei messaggi da inviare al gateway Vola
        ''' </summary>
        ''' <param name="Gsm">Lista dei destinatari del messaggio (Numero Gsm)</param>
        ''' <param name="Testo">Testo del messaggio e NON pu contenere i caratteti 'tab' e 'cr_lf'.
        ''' Se il testo del messaggio supera i 160 caratteri il gateway di VolaSMS effettua uno split del testo
        ''' in modo che ad ogni destinatario vengano inviati N messaggi ciascuno lungo al massimo 160 caratteri.
        ''' Il numero massimo di caratteri che pu contenere il campo messaggio e' 1440 caratteri.</param>
        ''' <param name="Mittente">Mittente del gruppo di invio. E' il mittente con cui verra' inviato l'sms al destinatario o ai destinatari del messaggio.
        ''' Il campo avr effetto solo se l'account utilizzata supporta l'invio di messaggi con campo mittente modificabile.
        ''' In quest'ultimo caso il gateway di VolaSMS utilizzer il valore di default vola.it o SMS, nel caso in cui il profilo di listino impostato lo supporti.
        ''' Il campo pu contenere al max 11 caratteri alfanumerici, oppure 14 caratteri numerici.
        ''' Nel caso in cui il campo non sia correttamente valorizzato il gateway restituir un errore.</param>
        ''' <returns>Il messaggio inserito nella lista dei messaggi da inviare al gateway Vola</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function AddMessageToSend(ByVal Gsm As String(), ByVal Testo As String, ByVal Mittente As String) As VolaMessaggio
            Return AddMessageToSend(Gsm, Testo, Mittente, DateTime.MinValue)
        End Function

        ''' <summary>
        ''' Inserisce un nuovo messaggio nella lista dei messaggi da inviare al gateway Vola
        ''' </summary>
        ''' <param name="Gsm">Lista dei destinatari del messaggio (Numero Gsm)</param>
        ''' <param name="Testo">Testo del messaggio e NON pu contenere i caratteti 'tab' e 'cr_lf'.
        ''' Se il testo del messaggio supera i 160 caratteri il gateway di VolaSMS effettua uno split del testo
        ''' in modo che ad ogni destinatario vengano inviati N messaggi ciascuno lungo al massimo 160 caratteri.
        ''' Il numero massimo di caratteri che pu contenere il campo messaggio e' 1440 caratteri.</param>
        ''' <param name="Mittente">Mittente del gruppo di invio. E' il mittente con cui verra' inviato l'sms al destinatario o ai destinatari del messaggio.
        ''' Il campo avr effetto solo se l'account utilizzata supporta l'invio di messaggi con campo mittente modificabile.
        ''' In quest'ultimo caso il gateway di VolaSMS utilizzer il valore di default vola.it o SMS, nel caso in cui il profilo di listino impostato lo supporti.
        ''' Il campo pu contenere al max 11 caratteri alfanumerici, oppure 14 caratteri numerici.
        ''' Nel caso in cui il campo non sia correttamente valorizzato il gateway restituir un errore.</param>
        ''' <param name="DataOra">Data e ora in cui il messaggio deve essere inviato</param>
        ''' <returns>Il messaggio inserito nella lista dei messaggi da inviare al gateway Vola</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function AddMessageToSend(ByVal Gsm As String(), ByVal Testo As String, ByVal Mittente As String, ByVal DataOra As DateTime) As VolaMessaggio
            Try
                Dim GsmOK As VolaMsdn
                Dim TestoOK As String
                Dim MittenteOK As String
                Dim DataOraOK As String

                Dim stato As VolaMsdnStatus = VolaMsdnStatus.MSDN_INLIST

                'Trasformo il carattere ritorno a capo
                TestoOK = Testo.Replace(vbCr & vbLf, vbLf)
                'Verifico caratteri non permessi
                Dim caract As String = VolaConstant.CaractTextOK
                Dim emailregex As New Regex(caract)
                Dim m As Match = emailregex.Match(Testo)
                If m.Success = False Then
                    stato = VolaMsdnStatus.MSDN_ERRTEXT
                End If

                'Verifico se il testo  troppo lungo considerando i caratteri che valgono doppio
                Dim buffer As String = TestoOK
                Dim CaratDouble As String() = VolaConstant.CaractDouble.Split(","c)
                For i As Integer = 0 To CaratDouble.Length - 1
                    buffer = buffer.Replace(CaratDouble(i), "xx")
                Next

                If LongMessage Then
                    If buffer.Length > 1440 Then
                        stato = VolaMsdnStatus.MSDN_ERRTEXT
                    End If
                Else
                    If buffer.Length > 160 Then
                        stato = VolaMsdnStatus.MSDN_ERRTEXT
                    End If
                End If

                'Verifico il mittente
                MittenteOK = Mittente
                If MittenteOK.Length < 1 OrElse MittenteOK.Length > 15 Then
                    stato = VolaMsdnStatus.MSDN_ERRSENDER
                End If

                'Verifico la data
                If DataOra = DateTime.MinValue OrElse DataOra = DateTime.MaxValue Then
                    DataOraOK = "0000-00-00" & VolaConstant.SeparatorBlock & "00:00"
                Else
                    DataOraOK = DataOra.ToString("yyyy-MM-dd") + VolaConstant.SeparatorBlock + DataOra.ToString("HH:mm")
                End If
                DataOraOK = DataOraOK.Replace(".", ":")

                'Verifico la capacit della lista dei messaggi
                If _MessageListToSend.Count >= 1000 Then
                    stato = VolaMsdnStatus.MSDN_ERRDATETIME
                End If

                'Creo un nuovo messaggio da inviare
                Dim NuovoMessaggio As New VolaMessaggio()

                'Verifico i destinari
                For i As Integer = 0 To Gsm.Length - 1
                    GsmOK = New VolaMsdn()
                    GsmOK.gsm = Gsm(i)
                    GsmOK.gsm = VolaCheckGsm.CheckGSMNumber(GsmOK.gsm, VolaCheckGsm.FormatGsm.PLUS_COUNTRY_MSDN)
                    If GsmOK.gsm.Length = 0 Then
                        GsmOK.status = VolaMsdnStatus.MSDN_ERRPHONEGSM
                    Else
                        GsmOK.status = stato
                    End If
                    NuovoMessaggio.listaDestinatari.Add(GsmOK)
                Next

                'Inizializzo il messaggio
                NuovoMessaggio.testo = TestoOK
                NuovoMessaggio.mittente = MittenteOK
                NuovoMessaggio.dataora = DataOraOK
                NuovoMessaggio.cid = _MessageListToSend.Count

                'Aggiungo il nuovo messaggio alla lista
                _MessageListToSend.Add(NuovoMessaggio)
                Return NuovoMessaggio
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try


        End Function

        ''' <summary>
        ''' Rimuove un messaggio dalla lista dei messaggi da inviare al gateway Vola
        ''' </summary>
        ''' <param name="index">Indice del messaggio da rimuovere</param>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        Public Function RemoveMessageToSend(ByVal index As Integer) As Boolean
            Try
                If index >= _MessageListToSend.Count Then
                    Return False
                End If

                'Rimuove il messaggio dalla lista
                _MessageListToSend.RemoveAt(index)
                Return True
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try

        End Function

        ''' <summary>
        ''' Svuota la lista dei messaggi da inviare al gateway Vola
        ''' </summary>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        Public Function ClearMessageToSend() As Boolean
            Try
                'Cancella tutta la lista
                _MessageListToSend.Clear()
                Return True
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

        ''' <summary>
        ''' Restituisce il numero di messaggi contenuti nella lista dei messaggi da inviare al gateway Vola
        ''' </summary>
        ''' <returns>Il numero di messaggi contenuti nella lista dei messaggi da inviare al gateway Vola</returns>
        Public Function CountMessageToSend() As Integer
            Try
                'Numero messaggi nella lista
                Return _MessageListToSend.Count
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

        ''' <summary>
        ''' Restituisce il messaggio inserito nella lista dei messaggi da inviare al gateway Vola alla posizione indicata
        ''' </summary>
        ''' <param name="index">Indice del messaggio nella lista</param>
        ''' <returns>Il messaggio inserito nella lista dei messaggi da inviare al gateway Vola alla posizione indicata</returns>
        Public Function GetMessageToSend(ByVal index As Integer) As VolaMessaggio
            Try
                Dim Lista As IEnumerator = _MessageListToSend.GetEnumerator(index, 1)
                Lista.MoveNext()
                Return DirectCast(Lista.Current, VolaMessaggio)
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

        ''' <summary>
        ''' Comando Vola consente l'invio di messaggi multipli in una unica richiesta, diminuendo cosi' il numero di connessioni tra client e server ed aumentando la performance di comunicazione.
        ''' </summary>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function SendMessageToSend() As Boolean
            Try

                Dim NuovoMessaggio As VolaMessaggio
                Dim NuovoMsdn As VolaMsdn
                Dim Risposta As String

                'Controllo se la lista  vuota
                If _MessageListToSend.Count = 0 Then
                    Return False
                End If

                'Make the url request
                Dim urlPath As String = (("CMD=44&UID=" & VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) & "&PWD=") + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) & "&SERIAL=") + Vola.VolaSmsPlus.VolaConstant.SerialCode
                If _VapCode.Length > 0 Then
                    urlPath = (urlPath & "&VCODE=") + _VapCode
                End If
                If _Notify Then
                    urlPath = urlPath & "&NOTIFY=1"
                Else
                    urlPath = urlPath & "&NOTIFY=0"
                End If
                If _TestMode Then
                    urlPath = urlPath & "&TEST=1"
                Else
                    urlPath = urlPath & "&TEST=0"
                End If
                urlPath = urlPath & "&SENDDATA="

                'Ciclo sui messaggi nella lista
                Dim Lista As IEnumerator = _MessageListToSend.GetEnumerator()
                While Lista.MoveNext()

                    NuovoMessaggio = DirectCast(Lista.Current, VolaMessaggio)
                    urlPath = urlPath + NuovoMessaggio.cid.ToString().Trim() + VolaConstant.SeparatorBlock
                    urlPath = urlPath + VolaUtility.UrlEncode(NuovoMessaggio.mittente) + VolaConstant.SeparatorBlock

                    'Ciclo sui destinatari del messaggio
                    Dim ListaMsdn As IEnumerator = NuovoMessaggio.listaDestinatari.GetEnumerator()
                    While ListaMsdn.MoveNext()
                        NuovoMsdn = DirectCast(ListaMsdn.Current, VolaMsdn)
                        If NuovoMsdn.status = VolaMsdnStatus.MSDN_INLIST Then
                            urlPath = urlPath + VolaUtility.UrlEncode(NuovoMsdn.gsm) + VolaConstant.SeparatorPhone
                        End If
                    End While
                    'Elimino l'ultmi due caratteri
                    If urlPath.Substring(urlPath.Length - 1, 1) = VolaConstant.SeparatorPhone Then
                        urlPath = urlPath.Substring(0, urlPath.Length - 1)
                    End If

                    'Aggiunfo gli altri parametri del messaggio
                    urlPath = urlPath + VolaConstant.SeparatorBlock
                    Dim e As New UTF8Encoding()
                    urlPath = urlPath + VolaUtility.UrlEncode(NuovoMessaggio.testo) + VolaConstant.SeparatorBlock
                    urlPath = urlPath + VolaUtility.UrlEncode(NuovoMessaggio.dataora)
                    urlPath = urlPath + VolaConstant.SeparatorGroup
                End While

                'Elimino l'ultmi due caratteri
                urlPath.Substring(0, urlPath.Length - 2)

                'Invia richiesta
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _
                _ProxyPassword)

                'Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta)

                Dim sListaOrderdID As String = ""
                Dim messaggio As String()
                Dim messaggi As String() = Risposta.Split(VolaConstant.SeparatorRespost1)
                'Aggiorno lo stato dei messaggi con la risposta del gateway
                For i As Integer = 0 To messaggi.Length - 1
                    messaggio = messaggi(i).Split(VolaConstant.SeparatorRespost2)
                    If messaggio(1).Length > 0 AndAlso messaggio(1) <> "0" Then
                        sListaOrderdID = sListaOrderdID + messaggio(1).Replace(",", ";") & ";"
                    End If

                    SetStatusMessage(messaggio(0), messaggio(2), messaggio(1))
                Next

                'Invio la conferma al gateway sull'invio avvenuto in modo corretto
                If sListaOrderdID.Length > 0 Then
                    Return ConfirmSendMessage(sListaOrderdID)
                Else
                    Return True
                End If

            Catch ex As VolaException
                Throw New VolaException(ex.Messaggio, ex.Codice)
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try

        End Function

        ''' <summary>
        ''' Aggiorno lo stato dei messaggi con la risposta del gateway
        ''' </summary>
        ''' <param name="sCID">ID identificativo</param>
        ''' <param name="PhoneGsmKO">Numero GSM del destinatario</param>
        ''' <param name="sOrderId">ID identificativo</param>
        Private Sub SetStatusMessage(ByVal sCID As String, ByVal PhoneGsmKO As String, ByVal sOrderId As String)
            Dim NuovoMsdn As VolaMsdn
            Dim CID As Integer = Int32.Parse(sCID)

            Dim Messaggio As VolaMessaggio = DirectCast(_MessageListToSend(CID), VolaMessaggio)
            Messaggio.orderid = sOrderId.Split(VolaConstant.SeparatorRespost3)

            Dim ListaMsdn As IEnumerator = Messaggio.listaDestinatari.GetEnumerator()
            While ListaMsdn.MoveNext()
                NuovoMsdn = DirectCast(ListaMsdn.Current, VolaMsdn)

                If PhoneGsmKO.IndexOf(NuovoMsdn.gsm) = -1 Then
                    NuovoMsdn.status = VolaMsdnStatus.MSDN_SENT
                Else
                    NuovoMsdn.status = VolaMsdnStatus.MSDN_ERRGATEWAY
                End If
            End While
        End Sub

        ''' <summary>
        ''' Comando Vola per la conferma dell'invio dei messaggi contenuti nalla lista dei messaggi da inviare al gateway Vola
        ''' </summary>
        ''' <param name="sListaOrderID">Lista ID identificati dei messaggi inviati correttamente</param>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Private Function ConfirmSendMessage(ByVal sListaOrderID As String) As Boolean
            Dim Risposta As String = ""

            Try
                'Make the url request
                Dim urlPath As String = ((("CMD=45&UID=" & VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) & "&PWD=") + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) & "&SERIAL=") + Vola.VolaSmsPlus.VolaConstant.SerialCode & "&ORDERID=") + sListaOrderID

                If _VapCode.Length > 0 Then
                    urlPath = (urlPath & "&VCODE=") + _VapCode
                End If

                'Invia richiesta
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _
                _ProxyPassword)

                'Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta)

                Return True
            Catch ex As VolaException
                Throw New VolaException(ex.Messaggio, ex.Codice)
            Catch ex As Exception
                Throw New VolaException("Errore grave durante la connessione al servizio SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function
#End Region

#Region "Metodi per la Gestione delle notifiche"
        ''' <summary>
        ''' Lista dei messaggi per ricevere lo stato di notifica
        ''' </summary>
        Public Property MessageListToNotify() As ArrayList
            Get
                Return _MessageListToNotify
            End Get
            Set(ByVal value As ArrayList)
                _MessageListToNotify = value
            End Set
        End Property

        ''' <summary>
        ''' Inserisce un nuovo messaggio nella lista dei messaggi per ricevere lo stato di notifica
        ''' </summary>
        ''' <param name="OrderId">Numero identificativo sms</param>
        ''' <returns>Il messaggio inserito nella lista</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function AddMessageToNotify(ByVal OrderId As String) As VolaMessaggio
            Return AddMessageToNotify(OrderId, "")
        End Function

        ''' <summary>
        ''' Inserisce un nuovo messaggio nella lista dei messaggi per ricevere lo stato di notifica
        ''' </summary>
        ''' <param name="OrderId">Numero identificativo sms</param>
        ''' <param name="Gsm">Destinatario del messaggio (Numero Gsm)</param>
        ''' <returns>Il messaggio inserito nella lista</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function AddMessageToNotify(ByVal OrderId As String, ByVal Gsm As String) As VolaMessaggio
            Try
                Dim GsmOK As New VolaMsdn()
                Dim OrderIdOK As String

                If OrderId.Length = 0 AndAlso OrderId.Length < 10 Then
                    Return Nothing
                End If
                OrderIdOK = OrderId

                'Verifico il destinario
                GsmOK.gsm = Gsm
                GsmOK.gsm = VolaCheckGsm.CheckGSMNumber(GsmOK.gsm, VolaCheckGsm.FormatGsm.PLUS_COUNTRY_MSDN)
                GsmOK.status = VolaMsdnStatus.MSDN_INLIST

                'Verifico la capacit della lista dei messaggi
                If _MessageListToNotify.Count >= 1000 Then
                    Return Nothing
                End If

                'Creo un nuovo messaggio da notificare
                Dim NuovoMessaggio As New VolaMessaggio()
                NuovoMessaggio.listaDestinatari.Add(GsmOK)
                NuovoMessaggio.orderid = New String(0) {}
                NuovoMessaggio.orderid(0) = OrderIdOK

                'Aggiungo il nuovo messaggio alla lista
                _MessageListToNotify.Add(NuovoMessaggio)
                Return NuovoMessaggio
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try

        End Function

        ''' <summary>
        ''' Rimuove un messaggio dalla lista dei messaggi per ricevere lo stato di notifica
        ''' </summary>
        ''' <param name="index">Indice del messaggio da rimuovere</param>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        Public Function RemoveMessageToNotify(ByVal index As Integer) As Boolean
            Try
                If index >= _MessageListToNotify.Count Then
                    Return False
                End If

                'Rimuove il messaggio dalla lista
                _MessageListToNotify.RemoveAt(index)
                Return True
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try

        End Function

        ''' <summary>
        ''' Svuota la lista dei messaggi per ricevere lo stato di notifica
        ''' </summary>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        Public Function ClearMessageToNotify() As Boolean
            Try
                'Cancella tutta la lista
                _MessageListToNotify.Clear()
                Return True
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

        ''' <summary>
        ''' Restituisce il numero di messaggi contenuti nella lista per ricevere lo stato di notifica
        ''' </summary>
        ''' <returns>Il numero di messaggi contenuti nella lista dei messaggi per ricevere lo stato di notifica</returns>
        Public Function CountMessageToNotify() As Integer
            Try
                'Numero messaggi nella lista
                Return _MessageListToNotify.Count
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

        ''' <summary>
        ''' Restituisce il messaggio inserito nella lista dei messaggi per ricevere lo stato di notifica alla posizione indicata
        ''' </summary>
        ''' <param name="index">Indice del messaggio nella lista</param>
        ''' <returns>Il messaggio inserito nella lista dei messaggi per ricevere lo stato di notifica alla posizione indicata</returns>
        Public Function GetMessageToNotify(ByVal index As Integer) As VolaMessaggio
            Try
                Dim Lista As IEnumerator = _MessageListToNotify.GetEnumerator(index, 1)
                Lista.MoveNext()
                Return DirectCast(Lista.Current, VolaMessaggio)
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

        ''' <summary>
        ''' Comando Vola che permette di conoscere lo status di invio di tutti gli sms nel caso in cui l'account utilizzata abbia abilitata la funzionalit di invio con notifica.
        ''' In caso contrario lo stato di notifica restituito si riferisce alla sola presa in consegna del messaggio lato gateway VolaSMS.
        ''' Se il comando va a buon fine restituisce una lista contenente lo stato informativo di ricezione del messaggio.
        ''' Questo tipo di comando permette di inviare al gateway pi richieste di notifica contemporaneamente.
        ''' </summary>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function GetStatusNotify() As Boolean

            Try

                Dim NuovoMessaggio As VolaMessaggio
                Dim NuovoMsdn As VolaMsdn
                Dim Risposta As String

                'Verfico che la lista non sia vuota
                If _MessageListToNotify.Count = 0 Then
                    Return False
                End If

                'Make the url request
                Dim urlPath As String = (("CMD=10&UID=" & VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) & "&PWD=") + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) & "&SERIAL=") + Vola.VolaSmsPlus.VolaConstant.SerialCode
                If _VapCode.Length > 0 Then
                    urlPath = (urlPath & "&VCODE=") + _VapCode
                End If

                urlPath = urlPath & "&QUERYDATA="

                'Ciclo per i messaggi nella lista
                Dim Lista As IEnumerator = _MessageListToNotify.GetEnumerator()
                While Lista.MoveNext()
                    NuovoMessaggio = DirectCast(Lista.Current, VolaMessaggio)

                    Dim ListaMsdn As IEnumerator = NuovoMessaggio.listaDestinatari.GetEnumerator()
                    ListaMsdn.MoveNext()

                    NuovoMsdn = DirectCast(ListaMsdn.Current, VolaMsdn)
                    urlPath = urlPath + NuovoMessaggio.orderid(0) + VolaConstant.SeparatorNotify1
                    urlPath = urlPath + VolaUtility.UrlEncode(NuovoMsdn.gsm) + VolaConstant.SeparatorNotify2
                End While

                'Elimino l'ultimo due caratteri
                urlPath = urlPath.Substring(0, urlPath.Length - 1)

                'Invia richiesta
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _
                _ProxyPassword)

                'Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta)
                If Risposta = "" OrElse Risposta = "NULL" Then
                    Return False
                End If

                Dim Find As Boolean
                Dim ListaMessaggi As String()
                Dim Messaggio As String()
                Dim Orderid As String()
                Dim ListaOrderid As String() = Risposta.Split(VolaConstant.SeparatorResNotify1.ToCharArray())
                Lista = _MessageListToNotify.GetEnumerator()
                'Ciclio per orderId (ricevuti)
                For i As Integer = 0 To ListaOrderid.Length - 1
                    Orderid = ListaOrderid(i).Split(VolaConstant.SeparatorResNotify2)
                    If Orderid.Length = 2 Then
                        If Orderid(1) <> "" AndAlso Orderid(1) <> "NULL" Then
                            Lista.Reset()
                            Find = True
                            While Lista.MoveNext() AndAlso Find
                                NuovoMessaggio = DirectCast(Lista.Current, VolaMessaggio)
                                If NuovoMessaggio.orderid(0) = Orderid(0) Then
                                    ListaMessaggi = Orderid(1).Split(VolaConstant.SeparatorResNotify3)
                                    NuovoMessaggio.listaDestinatari.Clear()
                                    For count As Integer = 0 To ListaMessaggi.Length - 1
                                        Messaggio = ListaMessaggi(count).Split(VolaConstant.SeparatorResNotify4)
                                        NuovoMsdn = New VolaMsdn()
                                        NuovoMsdn.datatimestatus = (Messaggio(0) & " ") + Messaggio(1)
                                        NuovoMsdn.gsm = Messaggio(2)
                                        Select Case Messaggio(3)
                                            Case "1"
                                                NuovoMsdn.status = VolaMsdnStatus.MSDN_SENT
                                                Exit Select
                                            Case "2"
                                                NuovoMsdn.status = VolaMsdnStatus.MSDN_SMSC
                                                Exit Select
                                            Case "3"
                                                NuovoMsdn.status = VolaMsdnStatus.MSDN_DELIVERED
                                                Exit Select
                                            Case "4"
                                                NuovoMsdn.status = VolaMsdnStatus.MSDN_ERROPERATOR

                                                Select Case Messaggio(4)
                                                    Case "000"
                                                        NuovoMsdn.reasoncode = VolaMsdnReasonCode.REASONCODE_UNKNOW
                                                        Exit Select
                                                    Case "101", "108"
                                                        NuovoMsdn.reasoncode = VolaMsdnReasonCode.REASONCODE_ERROR_DELIVERY
                                                        Exit Select
                                                    Case "103"
                                                        NuovoMsdn.reasoncode = VolaMsdnReasonCode.REASONCODE_LOK_OPERATOR
                                                        Exit Select
                                                    Case "111"
                                                        NuovoMsdn.reasoncode = VolaMsdnReasonCode.REASONCODE_LOK_USER
                                                        Exit Select
                                                    Case "120"
                                                        NuovoMsdn.reasoncode = VolaMsdnReasonCode.REASONCODE_SIM_FULL
                                                        Exit Select
                                                    Case "123"
                                                        NuovoMsdn.reasoncode = VolaMsdnReasonCode.REASONCODE_ERROR_OPERATOR
                                                        Exit Select
                                                    Case Else
                                                        NuovoMsdn.reasoncode = VolaMsdnReasonCode.REASONCODE_ERROR_DELIVERY
                                                        Exit Select
                                                End Select
                                                Exit Select
                                            Case Else
                                                NuovoMsdn.status = VolaMsdnStatus.MSDN_SENT
                                                Exit Select

                                        End Select
                                        
                                        NuovoMessaggio.listaDestinatari.Add(NuovoMsdn)
                                    Next
                                    Find = False
                                End If
                            End While
                        End If
                    End If
                Next
                Return True

            Catch ex As VolaException
                Throw New VolaException(ex.Messaggio, ex.Codice)
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

#End Region

#Region "Metodi per la ricezione dei messaggi e gestione chiavi di ricezione"
        ''' <summary>
        ''' Comando Vola che controlla se vi sono nuovi messaggi messaggi sul server e restituisce il numero di nuovi messaggi ricevuti.
        ''' </summary>
        ''' <returns>Il numero dei nuovi messaggi che ci sono sul server</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function CheckIncomingMessages() As Integer
            Return CheckIncomingMessages("")
        End Function

        ''' <summary>
        ''' Comando Vola che controlla se vi sono nuovi messaggi messaggi sul server e restituisce il numero di nuovi messaggi ricevuti.
        ''' </summary>
        ''' <param name="Key">La chiave di ricezione di cui si richiede il comando</param>
        ''' <returns>Il numero dei nuovi messaggi che ci sono sul server</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function CheckIncomingMessages(ByVal Key As String) As Integer

            Try
                'Make the url request
                Dim Risposta As String

                Dim urlPath As String = (("CMD=6&UID=" & VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) & "&PWD=") + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) & "&SERIAL=") + Vola.VolaSmsPlus.VolaConstant.SerialCode
                If Key.Length > 0 Then
                    urlPath = (urlPath & "&KEY=") + VolaUtility.UrlEncode(Key)
                End If
                If _VapCode.Length > 0 Then
                    urlPath = (urlPath & "&VCODE=") + _VapCode
                End If

                'Invia richiesta
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _
                _ProxyPassword)

                'Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta)

                If VolaUtility.IsInt(Risposta) Then
                    Return Integer.Parse(Risposta)
                Else
                    Return -1
                End If

            Catch ex As VolaException
                Throw New VolaException(ex.Messaggio, ex.Codice)
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

        ''' <summary>
        ''' Comando Vola che restituisce la lista dei nuovi messaggi ricevuti dallaccount dallultima esecuzione del comando andata a buon fine.
        ''' Se non vi sono nuovi messaggi la lista risulter vuota ed il comando restituir valore NULL.
        ''' </summary>
        ''' <returns>La lista dei nuovi messaggi ricevuti; NULL altrimenti</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function GetIncomingMessages() As ArrayList
            Return GetIncomingMessages("")
        End Function

        ''' <summary>
        ''' Comando Vola che restituisce la lista dei nuovi messaggi ricevuti dallaccount dallultima esecuzione del comando andata a buon fine.
        ''' La lista degli messaggi restituita dipende dal client applicativo che effettua la richiesta.
        ''' La discriminazione viene fatta in base al valore del parametro KEY (chiave di ricezione) inviato nella richiesta.
        ''' Se non vi sono nuovi messaggi la lista risulter vuota ed il comando restituir valore NULL.
        ''' </summary>
        ''' <param name="Key">La chiave di ricezione di cui si richiede il comando</param>
        ''' <returns>La lista dei nuovi messaggi ricevuti; NULL altrimenti</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function GetIncomingMessages(ByVal Key As String) As ArrayList

            Try
                'Make the url request
                Dim Risposta As String

                Dim urlPath As String = (("CMD=4&UID=" & VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) & "&PWD=") + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) & "&SERIAL=") + Vola.VolaSmsPlus.VolaConstant.SerialCode
                If Key.Length > 0 Then
                    urlPath = (urlPath & "&KEY=") + VolaUtility.UrlEncode(Key)
                End If
                If _VapCode.Length > 0 Then
                    urlPath = (urlPath & "&VCODE=") + _VapCode
                End If
                If _TestMode Then
                    urlPath = urlPath & "&TEST=1"
                Else
                    urlPath = urlPath & "&TEST=0"
                End If

                'Invia richiesta
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _
                _ProxyPassword)

                'Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta)

                Dim pos As Integer = Risposta.IndexOf(" "c)
                Dim num As String = Risposta.Substring(0, pos)
                Dim messaggi As String = Risposta.Substring(pos)

                If VolaUtility.IsInt(num) <> True Then
                    Return Nothing
                End If

                If Integer.Parse(num) = 0 OrElse messaggi.Length = 0 OrElse messaggi = "NULL" Then
                    Return Nothing
                End If

                Dim ListaRisposta As New ArrayList()
                Dim MessaggioVola As VolaMessaggio

                Dim rispostaSplit As String()
                rispostaSplit = Risposta.Split(New Char() {" "c}, 2)
                Risposta = rispostaSplit(1)

                Dim ListaMessaggi As String() = Risposta.Split(VolaConstant.SeparatorRecive1)
                'Ciclo sui messaggi ricevuti
                For i As Integer = 0 To ListaMessaggi.Length - 1
                    Dim Messaggio As String() = ListaMessaggi(i).Split(VolaConstant.SeparatorRecive2)
                    MessaggioVola = New VolaMessaggio()
                    MessaggioVola.dataora = (Messaggio(0) & " ") + Messaggio(1)
                    MessaggioVola.mittente = Messaggio(2)
                    MessaggioVola.testo = Messaggio(4)
                    MessaggioVola.key = Messaggio(3)

                    ListaRisposta.Add(MessaggioVola)
                Next

                Return ListaRisposta

            Catch ex As VolaException
                Throw New VolaException(ex.Messaggio, ex.Codice)
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

        ''' <summary>
        ''' Comando Vola che conferma al server la corretta ricezione dei messggi SMS effettuata mediante lultima esecuzione del comando GetIncomingMessages.
        ''' Questo comando deve essere eseguito dopo il comando GetIncomingMessages per confermare al gateway di VolaSMS l'avvenuta ricezione dei messaggi.
        ''' Se il comando non viene inviato, il gateway di VolaSMS restituir, alla ricezione del comando CMD=4, nuovamente l'elenco degli sms MO.
        ''' </summary>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function DelIncomingMessages() As Boolean
            Return DelIncomingMessages("")
        End Function

        ''' <summary>
        ''' Comando Vola che conferma al server la corretta ricezione dei messggi SMS effettuata mediante lultima esecuzione del comando GetIncomingMessages.
        ''' Questo comando deve essere eseguito dopo il comando GetIncomingMessages per confermare al gateway di VolaSMS l'avvenuta ricezione dei messaggi.
        ''' Se il comando non viene inviato, il gateway di VolaSMS restituir, alla ricezione del comando CMD=4, nuovamente l'elenco degli sms MO.
        ''' </summary>
        ''' <param name="Key">La chiave di ricezione di cui si richiede il comando</param>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function DelIncomingMessages(ByVal Key As String) As Boolean

            Try
                'Make the url request
                Dim Risposta As String

                Dim urlPath As String = (("CMD=5&UID=" & VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) & "&PWD=") + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) & "&SERIAL=") + Vola.VolaSmsPlus.VolaConstant.SerialCode
                If Key.Length > 0 Then
                    urlPath = (urlPath & "&KEY=") + VolaUtility.UrlEncode(Key)
                End If

                If _VapCode.Length > 0 Then
                    urlPath = (urlPath & "&VCODE=") + _VapCode
                End If

                'Invia richiesta
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _
                _ProxyPassword)

                'Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta)

                Return True

            Catch ex As VolaException
                Throw New VolaException(ex.Messaggio, ex.Codice)
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

        ''' <summary>
        ''' Comando Vola che restituisce il testo dell'autoresponder sulla chiave di ricezione
        ''' </summary>
        ''' <param name="Key">La chiave di ricezione di cui si richiede il comando</param>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function GetAutoResponder(ByVal Key As String) As String

            Try
                'Make the url request
                Dim Risposta As String

                Dim urlPath As String = ((("CMD=16&UID=" & VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) & "&PWD=") + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) & "&SERIAL=") + Vola.VolaSmsPlus.VolaConstant.SerialCode & "&KEY=") + VolaUtility.UrlEncode(Key)



                If _VapCode.Length > 0 Then
                    urlPath = (urlPath & "&VCODE=") + _VapCode
                End If

                'Invia richiesta
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _
                _ProxyPassword)

                'Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta)

                If Risposta = "AUTORESPONDER DISABLED!" Then
                    Risposta = ""
                End If

                Return Risposta

            Catch ex As VolaException
                Throw New VolaException(ex.Messaggio, ex.Codice)
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

        ''' <summary>
        ''' Comando Vola che imposta il testo dell'autoresponder sulla chiave di ricezione
        ''' </summary>
        ''' <param name="Key">La chiave di ricezione di cui si richiede il comando</param>
        ''' <param name="TestoAutoResponder">Il testo del nuovo messaggio di Autoresponder</param>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function SetAutoResponder(ByVal Key As String, ByVal TestoAutoResponder As String) As Boolean

            Try
                'Make the url request
                Dim Risposta As String

                Dim urlPath As String = (((("CMD=17&UID=" & VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) & "&PWD=") + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) & "&SERIAL=") + Vola.VolaSmsPlus.VolaConstant.SerialCode & "&KEY=") + VolaUtility.UrlEncode(Key) & "&AUTORESPONDER=") + VolaUtility.UrlEncode(TestoAutoResponder)

                If _VapCode.Length > 0 Then
                    urlPath = (urlPath & "&VCODE=") + _VapCode
                End If

                'Invia richiesta
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _
                _ProxyPassword)

                'Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta)

                Return True

            Catch ex As VolaException
                Throw New VolaException(ex.Messaggio, ex.Codice)
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function
#End Region

#Region "Metodi e propriet per la gestione dei mittenti"


        ''' <summary>
        ''' Lista dei mittenti associati all'account
        ''' </summary>
        Public Property ListSender() As ArrayList
            Get
                Return _ListSender
            End Get
            Set(ByVal value As ArrayList)
                _ListSender = value
            End Set
        End Property

        ''' <summary>
        ''' Svuota la lista dei mittenti, attezione non elimina i mittenti associati all'account
        ''' </summary>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        Public Function ClearListSender() As Boolean
            Try
                'Cancella tutta la lista
                _ListSender.Clear()
                Return True
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

        ''' <summary>
        ''' Restituisce il numero dei mittenti contenuti nella lista dei mittenti associati all'account
        ''' </summary>
        ''' <returns>Il numero di messaggi contenuti nella lista dei messaggi da inviare al gateway Vola</returns>
        Public Function CountListSender() As Integer
            Try
                'Numero messaggi nella lista
                Return _ListSender.Count
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

        ''' <summary>
        ''' Comando Vola che permette l'inserimento di un nuovo mittente alla lista dei mittenti dell'account Vola
        ''' </summary>
        ''' <param name="Sender">Il nuovo mittente da inserire; il campo pu contenere al max 11 caratteri alfanumerici
        ''' Nel caso in cui il campo non sia correttamente valorizzato il gateway restituir un errore.</param>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function AddSender(ByVal Sender As String) As Boolean
            Try

                Dim Risposta As String

                If Sender.Length < 1 Then
                    Return False
                End If

                'Se  un cellulare lo imposto nel formato corretto
                Dim SenderOK As String = Sender
                SenderOK = VolaCheckGsm.CheckGSMNumber(SenderOK, VolaCheckGsm.FormatGsm.EXISTCOUNTRY_MSDN)
                If SenderOK.Length = 0 Then
                    SenderOK = Sender
                End If

                'Azzerro la lista di cui occorrer leggere nuovavemte dal gateway
                _ListSender.Clear()

                'Make the url request
                Dim urlPath As String = ((("CMD=30&UID=" & VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) & "&PWD=") + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) & "&SERIAL=") + Vola.VolaSmsPlus.VolaConstant.SerialCode & "&SENDER=") + VolaUtility.UrlEncode(Sender)

                If _VapCode.Length > 0 Then
                    urlPath = (urlPath & "&VCODE=") + _VapCode
                End If

                'Invia richiesta
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _
                _ProxyPassword)

                'Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta)

                Return True

            Catch ex As VolaException
                Throw New VolaException(ex.Messaggio, ex.Codice)
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

        ''' <summary>
        ''' Comando Vola che permette l'inserimento di un nuovo mittente alla lista dei mittenti dell'account Vola
        ''' </summary>
        ''' <param name="Sender">Il nuovo mittente da inserire; il campo pu contenere al max 14 caratteri numerici
        ''' Nel caso in cui il campo non sia correttamente valorizzato il gateway restituir un errore.
        ''' Il mittente dovr essere autenticato attraverso la tecnica del PINCODE.
        ''' Lesecuzione del comando genera linvio di un SMS contente un pincode sul numero di cellulare da autenticare.</param>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function AddSenderGsm(ByVal Sender As String) As Integer

            Try
                'Make the url request
                Dim Risposta As String
                Dim SenderOK As String

                If Sender.Length < 1 Then
                    Return -1
                End If

                'Se non  un cellulare esco altrimenti lo imposto nel formato corretto
                SenderOK = Sender
                SenderOK = VolaCheckGsm.CheckGSMNumber(SenderOK, VolaCheckGsm.FormatGsm.EXISTCOUNTRY_MSDN)
                If SenderOK.Length = 0 Then
                    Return -1
                End If

                'Azzerro la lista di cui occorrer leggere nuovavemte dal gateway
                _ListSender.Clear()

                Dim urlPath As String = ((("CMD=34&UID=" & VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) & "&PWD=") + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) & "&SERIAL=") + Vola.VolaSmsPlus.VolaConstant.SerialCode & "&SENDER=") + VolaUtility.UrlEncode(SenderOK)

                If _VapCode.Length > 0 Then
                    urlPath = (urlPath & "&VCODE=") + _VapCode
                End If

                'Invia richiesta
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _
                _ProxyPassword)

                'Verifica risposta
                If Risposta = "42" Then
                    Return 0
                Else
                    Risposta = VolaUtility.GestioneRisposta(Risposta)
                End If

                Return 1

            Catch ex As VolaException
                Throw New VolaException(ex.Messaggio, ex.Codice)
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

        ''' <summary>
        ''' Comando Vola che permette la validazione del pincode per lautenticazione di un mittente (numero di cellulare)
        ''' </summary>
        ''' <param name="Sender">Il nuovo mittente da autenticare.</param>
        ''' <param name="Pincode">Il codice ricevuto tramite SMS sul numero di cellulare nonch il mittente da autenticare.</param>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function AuthenticateSender(ByVal Sender As String, ByVal Pincode As String) As Boolean
            Try
                'Make the url request
                Dim Risposta As String

                'Se non  un cellulare esco altrimenti lo imposto nel formato corretto
                Dim SenderOK As String = Sender
                SenderOK = VolaCheckGsm.CheckGSMNumber(SenderOK, VolaCheckGsm.FormatGsm.EXISTCOUNTRY_MSDN)
                If SenderOK.Length = 0 Then
                    Return False
                End If

                If Pincode.Length < 1 Then
                    Return False
                End If

                'Azzerro la lista di cui occorrer leggere nuovavemte dal gateway
                _ListSender.Clear()

                Dim urlPath As String = (((("CMD=33&UID=" & VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) & "&PWD=") + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) & "&SERIAL=") + Vola.VolaSmsPlus.VolaConstant.SerialCode & "&SENDER=") + VolaUtility.UrlEncode(SenderOK) & "&PINCODE=") + VolaUtility.UrlEncode(Pincode)

                If _VapCode.Length > 0 Then
                    urlPath = (urlPath & "&VCODE=") + _VapCode
                End If

                'Invia richiesta
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _
                _ProxyPassword)

                'Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta)

                Return True

            Catch ex As VolaException
                Throw New VolaException(ex.Messaggio, ex.Codice)
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

        ''' <summary>
        ''' Comando Vola che permette di cancellare un mittente alla lista dei mittenti dell'account Vola
        ''' </summary>
        ''' <param name="Sender">Il mittente da cancellare.</param>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function RemoveSender(ByVal Sender As String) As Boolean
            Try
                'Make the url request
                Dim Risposta As String

                'Se  un cellulare lo imposto nel formato corretto
                Dim SenderOK As String = Sender
                SenderOK = VolaCheckGsm.CheckGSMNumber(SenderOK, VolaCheckGsm.FormatGsm.EXISTCOUNTRY_MSDN)
                If SenderOK.Length = 0 Then
                    SenderOK = Sender
                End If

                'Azzerro la lista di cui occorrer leggere nuovavemte dal gateway
                _ListSender.Clear()

                Dim urlPath As String = ((("CMD=31&UID=" & VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) & "&PWD=") + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) & "&SERIAL=") + Vola.VolaSmsPlus.VolaConstant.SerialCode & "&SENDER=") + VolaUtility.UrlEncode(SenderOK)

                If _VapCode.Length > 0 Then
                    urlPath = (urlPath & "&VCODE=") + _VapCode
                End If

                'Invia richiesta
                Risposta = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _
                _ProxyPassword)

                'Verifica risposta
                Risposta = VolaUtility.GestioneRisposta(Risposta)

                Return True

            Catch ex As VolaException
                Throw New VolaException(ex.Messaggio, ex.Codice)
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

        ''' <summary>
        ''' Comando Vola che permette di inizializzre la lista dei mittenti Vola
        ''' </summary>
        ''' <returns>True in caso positivo, False altrimenti</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se il comando non ha un esito positivo</exception>
        ''' <exception cref="System.Exception">Generata in caso di errore non previsto</exception>
        Public Function GetListSender() As Boolean
            Try
                'Make the url request
                Dim PRlstSender As String
                _ListSender.Clear()

                Dim urlPath As String = (("CMD=32&UID=" & VolaUtility.UrlEncode(VolaUtility.MD5Encode(_UserName)) & "&PWD=") + VolaUtility.UrlEncode(VolaUtility.MD5Encode(_Password)) & "&SERIAL=") + Vola.VolaSmsPlus.VolaConstant.SerialCode


                If _VapCode.Length > 0 Then
                    urlPath = (urlPath & "&VCODE=") + _VapCode
                End If

                'Invia richiesta
                PRlstSender = VolaUtility.InvioDatiServer(urlPath, _EncryptSSL, _AutomaticProxy, _ProxyHost, _ProxyPort, _ProxyUsername, _
                _ProxyPassword)

                'Verifica risposta
                PRlstSender = VolaUtility.GestioneRisposta(PRlstSender)

                'Lista mittenti
                Dim DatiMitente As String()
                Dim NuovoMittente As VolaSender
                Dim ListaMitt As String() = PRlstSender.Split(VolaConstant.SeparatorSender1)
                For i As Integer = 0 To ListaMitt.Length - 1
                    DatiMitente = ListaMitt(i).Split(VolaConstant.SeparatorSender2)
                    NuovoMittente = New VolaSender()
                    NuovoMittente.mittente = DatiMitente(0)
                    Select Case DatiMitente(1)
                        Case "P"
                            NuovoMittente.type = VolaTypeSender.PREDEFINITO
                            Exit Select
                        Case "A"
                            NuovoMittente.type = VolaTypeSender.AUTENTICATO
                            Exit Select
                        Case "V"
                            NuovoMittente.type = VolaTypeSender.VOLA
                            Exit Select
                        Case "T"
                            NuovoMittente.type = VolaTypeSender.DA_AUTENTICARE
                            Exit Select
                        Case Else
                            NuovoMittente.type = VolaTypeSender.ERRATO
                            Exit Select

                    End Select
                    _ListSender.Add(NuovoMittente)
                Next

                Return True

            Catch ex As VolaException
                Throw New VolaException(ex.Messaggio, ex.Codice)
            Catch ex As Exception
                Throw New VolaException("Errore generico Vola SMS: " & ex.Message, VolaError.VOLASMSPLUS_ERROR_GENERIC)

            End Try
        End Function

#End Region

    End Class
End Namespace