'
'  Version: $RCSfile: VolaUtility.vb,v $ - $Revision: 1.2 $ - $Date: 2009/02/17 09:34:57 $
'
'  This file is part of VolaSmsPlus.
'
'   VolaSmsPlus is free software: you can redistribute it and/or modify
'   it under the terms of the GNU Lesser General Public License as published by
'   the Free Software Foundation, either version 3 of the License.
' 
'   VolaSmsPlus is distributed in the hope that it will be useful,
'   but WITHOUT ANY WARRANTY; without even the implied warranty of
'   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'   GNU Lesser General Public License for more details.
'
'   You should have received a copy of the GNU Lesser General Public License
'   along with VolaSmsPlus.  If not, see <http://www.gnu.org/licenses/>.
'

Imports System
Imports System.Net
Imports System.Text
Imports System.Web
Imports System.IO
Imports System.Security.Cryptography

Namespace Vola.VolaSmsPlus
    Friend Module VolaUtility
        ''' <summary>
        ''' Consente di trasformare il valore stringa di input e una stringa MD5 in output
        ''' </summary>
        ''' <param name="original">Valore stringa da trasformare</param>
        ''' <returns>Valore stringa trasformata in codice MD5</returns>
        ''' <exception cref="System.Exception">Generata se il valore stringa di input non  in un formato valido</exception>
        Friend Function MD5Encode(ByVal original As String) As String
            Try
                Dim originalBytes As Byte()
                Dim encodedBytes As Byte()
                Dim md5 As MD5

                md5 = New MD5CryptoServiceProvider()
                originalBytes = ASCIIEncoding.[Default].GetBytes(original)
                encodedBytes = md5.ComputeHash(originalBytes)

                'Convert encoded bytes back to a 'readable' string
                Return BitConverter.ToString(encodedBytes).Replace("-", "")
            Catch ex As Exception
                Throw ex
            End Try
        End Function

        ''' <summary>
        ''' Verifica se il valore stringa di input  un numero e restituisce
        ''' </summary>
        ''' <param name="valore">Valore stringa da verificare</param>
        ''' <returns>Ritorna True in caso positivo, False altrimenti</returns>
        ''' <exception cref="System.Exception">Generata se il valore stringa di input non  in un formato valido</exception>
        Friend Function IsInt(ByVal valore As String) As Boolean
            Try
                Integer.Parse(valore)
                Return True
            Catch
                Return False
            End Try
        End Function

        ''' <summary>
        ''' Metodo per inviare i comandi al gateway e per ricevere e gestire la risposta
        ''' </summary>
        ''' <param name="Dati">Valore stringa contente il comando nella sintassi Vola</param>
        ''' <param name="EncryptSSL">Valore booleano per il servizio di connessione sicura</param>
        ''' <param name="AutomaticProxy">Valore booleano per il servizio di proxy automatico</param>
        ''' <param name="ProxyHost">Valore stringa con il nome o l'indirizzo dell'host; solo in caso di AutomaticProxy non abilitato</param>
        ''' <param name="ProxyPort">Valore stringa con il numero di porta dell'host; solo in caso di AutomaticProxy non abilitato</param>
        ''' <param name="ProxyUsername">Valore stringa con la username dell'host; solo in caso di AutomaticProxy non abilitato</param>
        ''' <param name="ProxyPassword">Valore stringa con la password dell'host; solo in caso di AutomaticProxy non abilitato</param>
        ''' <returns>Valore stringa contente la risposta del Gateway Vola al comando inviato</returns>
        ''' <exception cref="System.UriFormatException">Generata se l'indirizzo gateway non  pi valido</exception>
        ''' <exception cref="SSystem.Net.WebException">Generata se si verifica un errore di connessione</exception>
        ''' <exception cref="System.Exception">Generata se i valori di input non sono in un formato valido</exception>
        Friend Function InvioDatiServer(ByVal Dati As String, ByVal EncryptSSL As Boolean, ByVal AutomaticProxy As Boolean, ByVal ProxyHost As String, ByVal ProxyPort As String, ByVal ProxyUsername As String, _
        ByVal ProxyPassword As String) As String
            Dim gatewayUrl As String
            Dim resp As HttpWebResponse = Nothing
            Dim Risposta As String
            Dim Proxy As WebProxy = Nothing

            'Tipo d connessione
            If EncryptSSL Then
                gatewayUrl = Vola.VolaSmsPlus.VolaConstant.HttpsVolaAddressGateway
            Else
                gatewayUrl = Vola.VolaSmsPlus.VolaConstant.HttpVolaAddressGateway
            End If

            Try
                'Crea oggetto per connessione
                Dim request As HttpWebRequest = DirectCast(WebRequest.Create(gatewayUrl), HttpWebRequest)

                'Gestione proxy
                If AutomaticProxy Then
                    request.Proxy = Nothing
                Else
                    Dim newUri As New Uri((Vola.VolaSmsPlus.VolaConstant.HttpProtocol + ProxyHost & ":") + ProxyPort)
                    Proxy = New WebProxy()

                    If ProxyUsername.Length > 0 Then
                        Proxy.Credentials = New NetworkCredential(ProxyUsername, ProxyPassword)
                    End If
                    Proxy.Address = newUri
                    request.Proxy = Proxy
                End If

                'Trasformo i dati stringa in byte
                Dim postDataBytes As Byte() = Encoding.UTF8.GetBytes(Dati)
                request.Method = Vola.VolaSmsPlus.VolaConstant.Methode
                request.ContentType = Vola.VolaSmsPlus.VolaConstant.ContentType
                request.ContentLength = postDataBytes.Length
                request.Timeout = 30000
                request.KeepAlive = True

                Dim requestStream As Stream = request.GetRequestStream()
                requestStream.Write(postDataBytes, 0, postDataBytes.Length)
                requestStream.Close()

                ' get response and write to console
                resp = DirectCast(request.GetResponse(), HttpWebResponse)
                Dim responseReader As New StreamReader(resp.GetResponseStream(), Encoding.UTF8)
                Risposta = responseReader.ReadToEnd()

                If Risposta.Length = 0 Then
                    Return (CInt(VolaError.VOLASMSPLUS_ERROR_NETWORK) & " Messaggio vuoto")
                End If
                resp.Close()
                Risposta = Risposta.Substring(16, Risposta.Length - 36)
                Return Risposta
            Catch e As System.Net.WebException
                Return ((CInt(VolaError.VOLASMSPLUS_ERROR_NETWORK) & " ") + e.Message)
            Catch e As System.UriFormatException
                Return ((CInt(VolaError.VOLASMSPLUS_ERROR_PROXY) & " ") + e.Message)
            Catch e As Exception
                Return ((CInt(VolaError.VOLASMSPLUS_ERROR_NETWORK) & " ") + e.Message)
            Finally
                If resp IsNot Nothing Then
                    resp.Close()
                End If
            End Try
        End Function

        ''' <summary>
        ''' Gestore di Errore di VolaAddress.ocx
        ''' </summary>
        ''' <param name="Risposta">Valore stringa contente la risposta dal gateway Vola comprensivo del codice di risposta</param>
        ''' <returns>Valore stringa con i dati di risposta dal gateway Vola senza il codice di risposta</returns>
        ''' <exception cref="Vola.VolaSmsPlus.VolaException">Generata se la risposta del gateway conteneva un codice di risposta diverso da 0</exception>
        Friend Function GestioneRisposta(ByVal Risposta As String) As String
            Dim Codice As VolaError = 0
            Dim Dati As String = ""
            Dim CodiceRisposta As String = ""
            Dim Messaggio As String = ""

            '
            CodiceRisposta = Risposta.Substring(0, 2)
            If IsInt(CodiceRisposta) Then
                Codice = DirectCast(Convert.ToInt32(CodiceRisposta), VolaError)
            Else
                Codice = VolaError.VOLASMSPLUS_ERROR_CGI
            End If

            Select Case Codice
                Case VolaError.VOLASMSPLUS_ERROR_GENERIC
                    'generic error
                    Messaggio = "Errore generico nel funzionamento di VolaSmsPlus. Contattare il supporto tecnico" & vbCr & vbLf
                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_NETWORK
                    'Network Error
                    Messaggio = "Si  verificato un problema di connessione ad Internet." & vbCr & vbLf & "Chiudere e riavviare l'applicazione. Se il problema persiste, controllare:" & vbCr & vbLf & vbTab & "- la connessione ad Internet" & vbCr & vbLf & ";" & vbTab & "- le impostazioni del Proxy Server (se esistente) dal Menu File|Opzioni.."
                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_AUTHENTICATION
                    'Authorization error
                    Messaggio = "Username o Password inserita errata. Riprovare." & vbCr & vbLf & "Si ricorda che i campi sono case sensitive, di conseguenza sia il nome utente che la password devono essere immessi con le lettere minuscole o maiuscole come sono state inserite in origine."

                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_CGI
                    'CGI Error
                    Messaggio = "Errore CGI. Sono stati passati parametri errati. Contattare il supporto tecnico."
                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_AUTORESPONDER
                    'Autoresponder error
                    Messaggio = "Errore nel testo dell'autoresponder o chiave di ricezione scaduta."
                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_CREDIT
                    'Credit Error
                    Messaggio = "Credito insufficiente. E' necessario acquistare una ricarica."
                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_COMMAND
                    'Command Error
                    Messaggio = "Errore di comunicazione col Gateway SMS; i parametri ricevuti hanno provocato un errore."
                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_GATEWAY
                    'Gateway Error
                    Messaggio = "Errore grave del gateway di Vola. Contattare il supporto tecnico!"
                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_VAPCODE
                    'Wrong VAPCode Error
                    Messaggio = "L'applicazione contiene un VAPCODE errato. Contattare il supporto tecnico."

                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_SENTSMS
                    Messaggio = "Si  tentato di inviare uno o pi messaggi uguali, sar inviato un solo messaggio di quelli uguali."
                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_WRONG
                    'General Error Gateway
                    Messaggio = "I dati inseriti non sono corretti per l'operazione da effettuare."
                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_SENDER_LIMIT_REACHED
                    'Sender Error
                    Messaggio = "Impossibile aggiungere nuovi mittenti poich si  raggiunto il limite massimo del numero dei mittenti impostabili per ogni singolo account. Per aggiungere un nuovo mittente  necessario prima cancellarne uno gi presente nella lista dei mittenti disponibili."
                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_SENDER_AUTH
                    'Sender Error
                    Messaggio = "Impossibile aggiungere il mittente poich il numero inserito richiede l'autenticazione con il codice di sblocco."
                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_SENDER_PINCODE
                    'Sender Error
                    Messaggio = "Il mittente che si  cercato di inserire non  stato aggiunto alla lista poich non  corretto."
                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_SENDER_PINCODE2
                    'Sender Error
                    Messaggio = "Si  verificato un problema per cui non  stato possibile inviare il pincode sul numero di cellulare da autenticare."
                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_SENDER_LIST_KO
                    'Sender Error
                    Messaggio = "Il mittente richiesto esiste gi nella lista dei mittenti impostati per laccount."
                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_SENDER_BLOCK
                    'Sender Error
                    Messaggio = "Il mittente richiesto non ha una lunghezza valida (11 caratteri alfanumerici oppure 16 numerici)"
                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_SENDER_INVALID
                    'Sender Error
                    Messaggio = "Non  acconsentito aggiungere il mittente richiesto."
                    Exit Select

                Case VolaError.VOLASMSPLUS_ERROR_PROXY
                    'Proxy Error
                    Messaggio = "ATTENZIONE, le impostazioni proxy sono errate, se non si conoscono informarmarsi presso il proprio amministratore di rete."
                    Exit Select

                Case VolaError.VOLASMSPLUS_OK
                    Messaggio = ""
                    If Risposta.Length <= 2 Then
                        Dati = ""
                    Else
                        Dati = Risposta.Substring(3)
                    End If
                    Exit Select
                Case Else
                    Messaggio = "Errore grave di comunicazione col server contattare il supporto tecnico."
                    Exit Select
            End Select

            If Messaggio.Length > 0 Then
                Throw New Vola.VolaSmsPlus.VolaException(Messaggio, Codice)
            End If
            Return Dati
        End Function


        Friend Function UrlEncode(ByVal text As String) As String
           
            Dim enc As String = text.Replace(" ", "(fb_space)")
            enc = enc.Replace("", "(fb_euro)")
            enc = HttpUtility.UrlEncode(enc, System.Text.Encoding.Unicode)
            enc = enc.Replace("%00", "")
            enc = GetUpEncodedString(enc)
            enc = enc.Replace("(fb_space)", "%20")
            enc = enc.Replace("(fb_euro)", "%80")
            Return enc
        End Function

        Friend Function GetUpEncodedString(ByVal strVal As String) As String
            Dim intPos As Integer = 0
            Dim strRet As String = ""

            While intPos <> -1 AndAlso strVal <> ""
                intPos = strVal.IndexOf("%", intPos + 1)
                If intPos <> -1 AndAlso intPos <= strVal.Length - 2 Then
                    strRet = strRet + strVal.Substring(strRet.Length, intPos - strRet.Length) + strVal.Substring(intPos, 3).ToUpper()
                End If
            End While
            If strRet = "" Then
                strRet = strVal
            Else
                strRet = strRet + strVal.Substring(strRet.Length)
            End If

            Return strRet
        End Function
    End Module


End Namespace

